C     Last change:  SC    9 Mar 2004   10:29 am
      PROGRAM BEALE22k
C
C       CALCULATES MODIFIED BEALE'S AND COOLEY'S MEASURES OF 
C       TOTAL, INTRINSIC, AND COMBINED INTRINSIC NONLINEARITY
C
C       PROGRAM BY R. L. COOLEY, USGS, DENVER, COLO.
C       MODIFIED FOR MODFLOW-2000 BY STEEN CHRISTENSEN, DEPT. OF EARTH
C         SCIENCES, UNIVERSITY OF AARHUS, DENMARK. 
C       USES PIECES FROM BEALE2K BY E.R. BANTA 8/12/1999
C         VARIABLES IN THE PARAMETER STATEMENT:
C          NDD = OR > SUM OF ALL OBSERVATIONS USED IN THE
C                     REGRESSION
C          NHD = OR > SUM OF THE HEAD OBSERVATIONS (NH IN LINE 5)
C          NDMHD = OR > SUM OF ALL OTHER OBSERVATIONS
C          NPD = OR > NUMBER OF PARAMETERS ESTIMATED IN THE REGRESSION
C          MPD = OR > MPR OF DATA LINE 5 OF MODFLOWP
C-------ASSIGN VERSION NUMBER AND DATE
      CHARACTER*40 VERSION
      PARAMETER (VERSION='1.0.0 31/10/2003')
      DOUBLE PRECISION VAR
      DOUBLE PRECISION, ALLOCATABLE :: C(:,:), G(:), SCLE(:), D(:)
      CHARACTER*10, ALLOCATABLE :: PARNAM(:)
      CHARACTER*12, ALLOCATABLE :: OBSNAM(:), GNAM(:)
      REAL, ALLOCATABLE :: BOPT(:), FOPT(:), B(:), F(:), GOPT(:),
     &          X(:,:), WH(:), WQ(:,:), WP(:), WG(:), Z(:,:),
     &          FOBS(:), PRM(:,:), WTP(:,:), BPRI(:), CC(:), SQCI(:),
     &          SQF0(:), SQG0(:), SQCIB(:), SQCIMX(:)
      INTEGER, ALLOCATABLE :: LN(:), NIPR(:)     
      INTEGER I, IIN1, IIN2, IMP, IOUT, IP, J, M, MPR, IPR, NO, ISTOP,
     &        NQ, NH, NOBS, NPD, NPTS, IDIF, ITYP,NPI, NQI, NHI, NOINT,
     &        NPE, LENGNAM, NONB_LEN, IOS, NQD, IPD, ISET
      REAL SQT, SQI, SCL, SQL, SUMQT, SUMQI, SUMQL, CR, BNT, BNI, VG,
     &     VP, VYP, UP, DG, GP, GPD, CI, QI, DET, SUMCI, SQLB, SUMQLB, 
     &     SUMCIB
      COMMON /ITP   / IIN1, IOUT
C
      CHARACTER*80 OUTNAM
      CHARACTER*84 FN
C
C**FORMAT LIST
  400 FORMAT(' ERROR OPENING FILE "',A,'" -- STOP EXECUTION')
  480 FORMAT (35X,'BEALE2-2k',//,
     &16X,'MODFLOW-2000 POST-PROCESSING PROGRAM TO CALCULATE',/,
     &8X,'MEASURES OF TOTAL, INTRINSIC, AND COMBINED INTRINSIC',
     &    ' NONLINEARITY',/,
     &29X,'Version ',A/)
  490 FORMAT(
     &' ERROR FINDING INPUT FILES.  OUTNAM MUST BE SPECIFIED AS A',/,
     &' STRING OTHER THAN "NONE" IN THE OBS FILE.  THE FILE',/,
     &' BEALE.ERR MAY CONTAIN ADDITIONAL DIAGNOSTIC INFORMATION')
  500 FORMAT (5I10,2F14.0)
  505 FORMAT (8F15.0)
  510 FORMAT (
     &' NUMBER OF ESTIMATED PARAMETERS...................:',I5,/, 
     &' NUMBER OF HEAD OBSERVATIONS......................:',I5,/, 
     &' NUMBER OF ALL OTHER OBSERVATIONS.................:',I5,/, 
     &' TOTAL NUMBER OF OBSERVATIONS.....................:',I5,//,
     &' NUMBER OF PRIOR-INFORMATION EQUATIONS............:',I5,/, 
     &' NUMBER OF PRIOR INFORMATION W/ FULL WEIGHT MATRIX:',I5,/, 
     &' NUMBER OF DATA SETS USED FOR BEALES MEASURE......:',I5,//,
     &' CALCULATED ERROR VARIANCE........................:',G12.5,//,
     &' CORRECTION FACTOR FOR CONFIDENCE REGION..........:',G12.5)
  515 FORMAT (/,23X,'OPTIMUM PARAMETERS',/,
     &4X,2('NO. NAME',10X,'BOPT',8X))
  520 FORMAT (/,6X,
     &        'DEPENDENT VARIABLES COMPUTED WITH OPTIMUM PARAMETERS',/,
     &        4X,2('NO. OBSERVATION',5X,'FOPT',8X))
  521 FORMAT (/,6X,
     &        'INTERVAL VARIABLES COMPUTED WITH OPTIMUM PARAMETERS',/,
     &        4X,2('NO.  INTERVAL  ',5X,'GOPT',8X))
  525 FORMAT (/,21X,'PARAMETERS FOR SAMPLE NO.',I4,/,4X,
     &        2('NO. NAME',12X,'B',9X))
  530 FORMAT (/,9X,'DEPENDENT VARIABLES COMPUTED FOR SAMPLE NO.',I4,/,
     &        4X,2('NO. OBSERVATION',6X,'FC',9X))
  535 FORMAT (/,' SENSITIVITIES FOR OPTIMUM PARAMETERS')
  555 FORMAT (/,11X,'RELIABILITY WEIGHTS FOR SAMPLE INFORMATION',/,4X,
     &        2('NO. OBSERVATION',6X,'W',10X))
  556 FORMAT (/,11X,'RELIABILITY WEIGHTS FOR INTERVAL SAMPLE',/,4X,
     &        2('NO.   INTERVAL ',6X,'W',10X))
  580 FORMAT (6F13.0)
  590 FORMAT (/,11X,'OBSERVED VALUES OF THE DEPENDENT VARIABLES',/,4X,
     &        2('NO. OBSERVATION',5X,'FOBS',8X))
  595 FORMAT (8F13.0)
  600 FORMAT (/,' MULTIPLE PRIOR NUMBER',I3,' ESTIMATE(LOG IF LN>0)=',
     &        G10.3,', WEIGHT=',G10.3)
  615 FORMAT (8I10)
  620 FORMAT (/,9X,'PARAMETER',/,3X,'NO.',5X,'NAME',7X,'LN')
  625 FORMAT (1X,I5,3X,A,3X,I2)
  630 FORMAT (6(A12,1X))
  635 FORMAT (' END OF BEALE2.DAT',/,'  MODFLOW-2000 SOLUTION MUST NOT',
     &        ' HAVE CONVERGED FOR NEXT SET OF PARAMETER VALUES.',/,
     &        ' CHECK PARAMETER VALUES; NEGATIVE VALUES CAN BE MADE TO',
     &        ' STAY POSITIVE BY ESTIMATING THE LOG TRANSFORM')
  636 FORMAT (' END OF BEALE4.DAT',/,'  MODFLOW-2000 SOLUTION MUST NOT',
     &        ' HAVE CONVERGED FOR NEXT SET OF PARAMETER VALUES.',/,
     &        ' CHECK PARAMETER VALUES; NEGATIVE VALUES CAN BE MADE TO',
     &        ' STAY POSITIVE BY ESTIMATING THE LOG TRANSFORM')
  645 FORMAT (6(A10,1X))
  660 FORMAT(/,' OBSERVATIONS WITH A FULL WEIGHT MATRIX:')
  670 FORMAT(/,' PRIOR INFORMATION WITH FULL WEIGHT MATRIX')
  680 FORMAT(/,'   FULL WEIGHT MATRIX:')
  690 FORMAT(3X,'PARAMETERS INVOLVED AND PRIOR VALUES:')
  700 FORMAT(5X,A,2X,1PG13.6)
  710 FORMAT(//,' TOTAL NONLINEARITY (BNT).......... = ',G11.5,
     &        /,' INTRINSIC NONLINEARITY (BNI)...... = ',G11.5,//)
  720 FORMAT(///,' ****** NONLINEARITY MEASURES FOR ',A,' INTERVALS')
  730 FORMAT(//,
     &' TOTAL NUMBER OF INTERVALS (NOINT)......................:',I5,/,
     &' NUMBER OF INTERVALS FOR HEAD (NHI).....................:',I5,/,
     &' NUMBER OF INTERVALS FOR HEAD DEPENDENT FLOWS (NQI).....:',I5,/,
     &' NUMBER OF INTERVALS FOR PARAMETERS (NPI)...............:',I5,/,
     &' INTERVAL TYPE (1=CONFIDENCE,2=PREDICTION)..............:',I5,/,
     &' HEAD AND FLOW INTERVALS ON DIFFERENCES (0,1)=(NO,YES)..:',I5)
  740 FORMAT (/,16X,
     &        'COMBINED INTRINSIC NONLINEARITY',/,
     &        4X,2('  INTERVAL NO. ',5X,' BMI',8X))
  750 FORMAT (/,6X,
     &        'COMBINED INTRINSIC NONLINEARITY AS IF F WERE LINEAR ',/,
     &        4X,2('  INTERVAL NO. ',5X,'BMF0',8X))
  760 FORMAT (/,6X,
     &        'COMBINED INTRINSIC NONLINEARITY AS IF G WERE LINEAR ',/,
     &        4X,2('  INTERVAL NO. ',5X,'BMG0',8X))
  770 FORMAT (/,11X,
     &        'COMBINED INTRINSIC NONLINEARITY - MAX. SUM',/,
     &        4X,2('  INTERVAL NO. ',4X,' BMIMAX',6X))
C
C     WRITE PROGRAM NAME AND VERSION TO SCREEN
      WRITE (*,*)
      WRITE (*,480) VERSION
C**DEFINE INPUT AND OUTPUT UNIT NUMBERS
      IIN1 = 11
      IIN2 = 2
      IOUT = 3
      OPEN (IOUT,FILE='BEALE.ERR',STATUS='UNKNOWN')
C     FIND INPUT FILE NAMES BY READING NAME AND OBS FILES,
C     AND READING OUTNAM
      CALL FINDINP(IOUT,OUTNAM,IIN1)
      IF (OUTNAM.EQ.'NONE') THEN
        WRITE(*,490)
        CLOSE(IOUT)
        STOP
      ELSE
        CLOSE(IOUT,STATUS='DELETE')
      ENDIF
      LENGNAM = NONB_LEN(OUTNAM,80)
C**OPEN FILES
      FN = OUTNAM(1:LENGNAM)//'.#be'
      OPEN (IOUT,FILE=FN,STATUS='UNKNOWN')
      WRITE (IOUT,480) VERSION
      FN = OUTNAM(1:LENGNAM)//'._b1'
      OPEN (IIN1,FILE=FN,STATUS='OLD',ACCESS='SEQUENTIAL',
     &      FORM='FORMATTED',ERR=2,IOSTAT=IOS)
    2 CONTINUE
      IF (IOS.NE.0) THEN
        LENGNAM = NONB_LEN(FN,84)
        WRITE (*,400) FN(1:LENGNAM)
        STOP
      ENDIF
      FN = OUTNAM(1:LENGNAM)//'._b2'
      OPEN (IIN2,FILE=FN,STATUS='OLD',ACCESS='SEQUENTIAL',
     &      FORM='FORMATTED',ERR=4,IOSTAT=IOS)
    4 CONTINUE
      IF (IOS.NE.0) THEN
        LENGNAM = NONB_LEN(FN,84)
        WRITE (*,400) FN(1:LENGNAM)
        STOP
      ENDIF
C**READ BASE DATA
      IPR = 0
C     READ ITEM 1 FROM _b1 FILE
      READ (IIN1,500) NPE, NOBS, NQ, MPR, IPR, VAR, CR
      NH = NOBS - NQ
      NPTS = 2*NPE
      IF (CR.LT.1.E-10) CR=0.
      WRITE (IOUT,510) NPE, NH, NQ, NOBS, MPR, IPR, NPTS, VAR, CR
C
C     ALLOCATE MEMORY FOR ARRAYS
      ALLOCATE(C(NPE,NPE),G(NPE),SCLE(NPE),PARNAM(NPE),BOPT(NPE),
     &         B(NPE),LN(NPE))
      ALLOCATE(OBSNAM(NOBS),FOPT(NOBS),F(NOBS),FOBS(NOBS),X(NPE,NOBS))
      ALLOCATE(WH(NH),WQ(NQ,NQ))
      ALLOCATE(WP(MPR),PRM(NPE+1,MPR))
      ALLOCATE(WTP(IPR,IPR),BPRI(IPR),NIPR(IPR))
      NQD=NQ
      IPD=IPR
      NPD=NPE
C
C     READ AND WRITE PARAMETER NAMES
C     READ ITEMS 2 AND 3 FROM _b1 FILE
      READ (IIN1,645) (PARNAM(I),I=1,NPE)
      READ (IIN1,580) (BOPT(J),J=1,NPE)
      WRITE (IOUT,515)
      CALL PRTOTB10(BOPT,PARNAM,NPE)
C     READ AND WRITE OBSERVATION NAMES AND SIMULATED VALUES
C     READ ITEMS 4 AND 5 FROM _b1 FILE
      READ (IIN1,630) (OBSNAM(I),I=1,NOBS)
      READ (IIN1,580) (FOPT(I),I=1,NOBS)
      WRITE (IOUT,520)
      CALL PRTOTB12(FOPT,OBSNAM,NOBS)
C     READ AND WRITE OBSERVED VALUES
C     READ ITEM 6 FROM _b1 FILE
      READ (IIN1,580) (FOBS(I),I=1,NOBS)
      WRITE (IOUT,590)
      CALL PRTOTB12(FOBS,OBSNAM,NOBS)
C     READ AND WRITE WEIGHTS FOR HEADS
      WRITE (IOUT,555)
      IF (NH.GT.0) THEN
C       READ ITEM 7 FROM _b1 FILE
        READ (IIN1,505) (WH(I),I=1,NH)
        WRITE (IOUT,*) ' HEADS:'
        CALL PRTOTB12(WH,OBSNAM,NH)
      ENDIF
C    READ AND WRITE FULL WEIGHT MATRIX ON OBSERVATIONS
      IF (NQ.GT.0) THEN
        DO 10 I = 1, NQ
C         READ ITEM 8 FROM _b1 FILE
          READ (IIN1,505) (WQ(I,J),J=1,NQ)
   10   CONTINUE
        WRITE (IOUT,660)
        CALL PRTOTD(WQ,NQ,NQ,NQD,OBSNAM,NH)
      ENDIF
C     READ AND WRITE SENSITIVITIES FOR OBSERVATIONS
      DO 20 J = 1, NOBS
C       READ ITEM 9 FROM _b1 FILE
        READ (IIN1,580) (X(I,J),I=1,NPE)
   20 CONTINUE
      WRITE (IOUT,535)
      CALL PRTOTX(X,NPE,NOBS,NPD,OBSNAM,PARNAM)
C     READ AND WRITE PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
        DO 30 IMP = 1, MPR
C         READ ITEM 10 FROM _b1 FILE
          READ (IIN1,505) (PRM(IP,IMP),IP=1,NPE+1), WP(IMP)
          WRITE (IOUT,600) IMP, PRM(NPE+1,IMP), WP(IMP)
          CALL PRTOTB10(PRM(1,IMP),PARNAM,NPE)
   30   CONTINUE
      ENDIF
C     READ AND WRITE PRIOR INFORMATION WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
        WRITE (IOUT,670)
C       READ ITEM 11 AND FIRST ITEM 12 FROM _b1 FILE
        READ (IIN1,615) (NIPR(I),I=1,IPR)
        READ (IIN1,580) (BPRI(I),I=1,IPR)
        WRITE (IOUT,690)
        WRITE (IOUT,700) (PARNAM(NIPR(I)),BPRI(I),I=1,IPR)
        DO 35 I = 1, IPR
C         READ SECOND ITEM 12 FROM _b1 FILE
          READ (IIN1,580) (WTP(IP,I),IP=1,IPR)
   35   CONTINUE
        WRITE (IOUT,680)
        CALL PRTOTP(WTP,IPR,IPR,IPD,PARNAM,NIPR,IPD,NPD)
      ENDIF
C     READ AND WRITE FLAGS INDICATING LOG TRANSFORMED PARAMETERS
C     READ ITEM 13 FROM _b1 FILE
      READ (IIN1,615) (LN(I),I=1,NPE)
      WRITE (IOUT,620)
      WRITE (IOUT,625) (I,PARNAM(I),LN(I),I=1,NPE)
      DO 40 IP = 1, NPE
        IF (LN(IP).NE.0) BOPT(IP) = ALOG(BOPT(IP))
   40 CONTINUE
C
C**COMPUTE C=X'*WGT*X AND FACTOR IT (THE COEFFICIENT MATRIX)
      CALL SETC(NPE,NOBS,NH,NQ,MPR,IPR,C,G,X,WH,WQ,PRM,WP,WTP,SCLE,NIPR,
     &          NPD)
C
C**READ DATA FOR EACH SAMPLE AND COMPUTE MODIFIED BEALE'S MEASURE FOR
C**TOTAL NONLINEARITY, BNT, AND INTRINSIC NONLINEARITY, BNI
      SQT=0.
      SQI=0.
      SQL=0.
      SQLB=0.
      DO 180 M = 1, NPTS
C        READ A PARAMETER SET
         READ (IIN2,595,END=190) (B(J),J=1,NPE)
         WRITE (IOUT,525) M
         CALL PRTOTB10(B,PARNAM,NPE)
         DO 50 I = 1, NPE
            IF (LN(I).NE.0) B(I) = ALOG(B(I))
   50    CONTINUE
C        READ VALUES SIMULATED WITH THIS PARAMETER SET
         READ (IIN2,580) (F(I),I=1,NOBS)
         WRITE (IOUT,530) M
         CALL PRTOTB12(F,OBSNAM,NOBS)
C
C**      COMPUTE G=X'*WGT*(F-(FOPT+X*(B-BOPT))) AND SOLVE FOR PARAMETER
C        CORRECTIONS SO AS OUTPUT G=C*G=PSI, F=F-(FOPT+X*(B-BOPT))
         CALL SETG(NPE,NOBS,NH,NQ,C,G,X,WH,WQ,SCLE,F,FOPT,B,BOPT,NPD,
     &             0,DG,VYP,GP,GP,GPD,G,X(1,1))
C
         SQT=SQT+SUMQT(NOBS,NH,NQ,F,WH,WQ)
         SQI=SQI+SUMQI(NPE,NOBS,NH,NQ,MPR,IPR,F,WH,WQ,X,G,PRM,WP,WTP,
     &                 NIPR)
         IF (CR.LT.1.E-10) THEN
            SQL=SQL+SUMQL(NPE,NOBS,NH,NQ,MPR,IPR,WH,WQ,X,PRM,WP,WTP,
     &                    NIPR,B,BOPT)**2
            SQLB=SQLB+SUMQLB(NPE,NOBS,NH,NQ,MPR,IPR,WH,WQ,X,G,PRM,WP,
     &                       WTP,NIPR,B,BOPT)**2
         END IF
  180 CONTINUE
C     END LOOP FOR DATA SETS; CALCULATE AND WRITE RESULTS
      IF (CR.LT.1.E-10) THEN
         SCL=NPE*VAR
C        IF THE FOLLOWING LINE IS COMMENTED OUT THE ORIGINAL NUMBERS ARE CALCULATED
         SCL=SQL/SCL
         BNT=SQT*SCL/SQL
         BNI=SQI*SCL/SQLB
      ELSE
         SCL=2.*NPE*NPE*CR*VAR
         BNT=SQT/SCL
         BNI=SQI/SCL
      END IF
      WRITE(IOUT,710) BNT,BNI
C
      GOTO 200
  190 WRITE (IOUT,635)
C
C***** NONLINEARUTY MEASURES FOR CONFIDENCE OR PREDICTION INTERVALS
  200 CONTINUE
      CLOSE(IIN1)
      CLOSE(IIN2)
      FN = OUTNAM(1:LENGNAM)//'._b3'
      OPEN (IIN1,FILE=FN,STATUS='OLD',ACCESS='SEQUENTIAL',
     &      FORM='FORMATTED',ERR=202,IOSTAT=IOS)
  202 CONTINUE
      IF (IOS.NE.0) THEN
        LENGNAM = NONB_LEN(FN,84)
        WRITE (*,400) FN(1:LENGNAM)
        STOP
      ENDIF
      FN = OUTNAM(1:LENGNAM)//'._b4'
      OPEN (IIN2,FILE=FN,STATUS='OLD',ACCESS='SEQUENTIAL',
     &      FORM='FORMATTED',ERR=204,IOSTAT=IOS)
  204 CONTINUE
      IF (IOS.NE.0) THEN
        LENGNAM = NONB_LEN(FN,84)
        WRITE (*,400) FN(1:LENGNAM)
        STOP
      ENDIF
C
C     READ BASE DATA
      READ (IIN1,'(6I10)') NOINT, NHI, NQI, NPI, ITYP, IDIF
      IF (ITYP.EQ.2) THEN
         WRITE(IOUT,720) 'PREDICTION'
      ELSE
         WRITE(IOUT,720) 'CONFIDENCE'
      END IF
      WRITE (IOUT,730) NOINT, NHI, NQI, NPI, ITYP, IDIF
      IF (NOINT.LT.1) GOTO 999
C
C     ALLOCATE MEMORY FOR ARRAYS
      ALLOCATE(GNAM(NOINT),GOPT(NOINT),WG(NOINT),Z(NPE,NOINT),
     &         CC(NOINT),D(NPE),SQCI(NOINT),SQF0(NOINT),SQG0(NOINT),
     &         SQCIB(NOINT),SQCIMX(NOINT))
C      
C     READ NAME AND VALUES OF PREDICTIONS COMPUTED WITH OPTIMUM VALUES
      READ (IIN1,630) (GNAM(I),I=1,NOINT)
      READ (IIN1,580) (GOPT(I),I=1,NOINT)
      WRITE (IOUT,521)
      CALL PRTOTB12(GOPT,GNAM,NOINT)
C     READ WEIGHTS OF PREDICTIONS
      READ(IIN1,505) (WG(I),I=1,NOINT)
      IF (ITYP.NE.2) THEN
         DO 208 I=1,NOINT
            WG(I)=0.
  208    CONTINUE       
      END IF   
      WRITE(IOUT,556)
      CALL PRTOTB12(WG,GNAM,NOINT)
C     READ SENSITIVITIES OF PREDICTIONS WITH RESPECT TO PARAMETERS
      DO 210 I=1,NOINT
         READ(IIN1,580) (Z(IP,I),IP=1,NPE)
  210 CONTINUE           
      WRITE (IOUT,535)
      CALL PRTOTX(Z,NPE,NOINT,NPD,GNAM,PARNAM)
C     READ CORRECTIONS FACTORS FOR CONFIDENCE OR PREDICTION INTERVALS
      READ(IIN1,'(6(1X,F13.0))') (CC(I),I=1,NOINT)
      DO 215 I=1,NOINT 
         IF (CC(I).LT.1.E-10) CC(I)=0.
  215 CONTINUE
      WRITE(IOUT,'(/,A)') ' CORRECTION FACTORS'
      CALL PRTOTB12(CC,GNAM,NOINT)
C      
C**READ DATA FOR EACH INTERVAL AND COMPUTE COOLEY MEASURES FOR
C**COMBINED INTRINSIC NONLINEARITY
C
      DO 220 I=1,NOINT
         SQCI(I)=0.
         SQF0(I)=0.
         SQG0(I)=0.
         SQCIB(I)=0.
         SQCIMX(I)=0.
  220 CONTINUE
      DO 300 NO=1,NOINT
         WRITE(IOUT,'(//,A,I3)') ' *** INTERVAL NO. ',NO
C
C        COMPUTE D=CZ=(X'*WGT*X)^-1*Z
         DO 230 IP=1,NPE
            D(IP)=Z(IP,NO)/SCLE(IP) 
  230    CONTINUE
         CALL SOLVE(C,D,D,NPE,NPD,0,ISTOP,DET)
C        COMPUTE VG=Q'Q AND VYP=Q'Q+WG^-1
         VG=0.
         DO 240 IP=1,NPE
            D(IP)=D(IP)/SCLE(IP) 
            VG=VG+D(IP)*Z(IP,NO)
  240    CONTINUE
         VP=0.
         IF (ITYP.EQ.2.AND.WG(NO).GT.0) VP=1./WG(NO)
         VYP=VG+VP
C         
         DO 290 ISET=1,2
C           READ A PARAMETER SET
            READ (IIN2,595,END=890) (B(J),J=1,NPE)
            WRITE (IOUT,525) ISET
            CALL PRTOTB10(B,PARNAM,NPE)
            DO 250 I = 1, NPE
               IF (LN(I).NE.0) B(I) = ALOG(B(I))
  250       CONTINUE
C           READ VALUES SIMULATED WITH THIS PARAMETER SET
            READ (IIN2,580) (F(I),I=1,NOBS)
            WRITE (IOUT,530) ISET
            CALL PRTOTB12(F,OBSNAM,NOBS)
C           READ VALUE OF PREDICTION CORRESPONDING TO THIS PARAMETER SET
            READ(IIN2,*) GP
            WRITE(IOUT,'(//,A,1X,G11.5)') ' PREDICTED VALUE, GP=',GP
C            
C**         COMPUTE G=C*G, F=F-(FOPT+X*(B-BOPT))
            CALL SETG(NPE,NOBS,NH,NQ,C,G,X,WH,WQ,SCLE,F,FOPT,B,BOPT,NPD,
     &                1,DG,VYP,GP,GOPT(NO),GPD,D,Z(1,NO))
            UP=VP*DG
            QI=SUMQI(NPE,NOBS,NH,NQ,MPR,IPR,F,WH,WQ,X,G,PRM,WP,WTP,
     &               NIPR)
            CI=SUMCI(NPE,NOBS,NH,NQ,MPR,IPR,F,WH,WQ,X,G,PRM,WP,WTP,
     &               NIPR,D)
            DG=UP-VP*GPD/VYP
            SQCI(NO)=SQCI(NO)+QI-GPD*(CI+CI-GPD)/VYP+DG*WG(NO)*DG
            SQF0(NO)=SQF0(NO)+GPD*GPD/VYP
            SQG0(NO)=SQG0(NO)+QI+UP*WG(NO)*UP
            IF (CC(NO).LT.1.E-10) THEN
               QI=SUMQLB(NPE,NOBS,NH,NQ,MPR,IPR,WH,WQ,X,G,PRM,WP,
     &                   WTP,NIPR,B,BOPT)
               CI=SUMCIB(NPE,NOBS,NH,NQ,MPR,IPR,WH,WQ,X,G,PRM,WP,WTP,
     &                    NIPR,D,B,BOPT)
               SQCIB(NO)=SQCIB(NO)+
     &                   (QI-GPD*(CI+CI-GPD)/VYP+DG*WG(NO)*DG)**2
            END IF    
  290    CONTINUE
  300 CONTINUE
C     END LOOP FOR INTERVALS; CALCULATE AND WRITE RESULTS
      DO 310 NO=1,NOINT
         IF (CC(NO).LT.1.E-10) THEN
            SCL=NPE*VAR
C           IF THE FOLLOWING LINE IS COMMENTED OUT THE ORIGINAL NUMBERS ARE CALCULATED
            SCL=SQL/SCL
            BNT=SQT*SCL/SQL
            BNI=SQI*SCL/SQLB
            SQCI(NO)=SQCI(NO)*SCL/SQCIB(NO)
            SQF0(NO)=SQF0(NO)*SCL/SQCIB(NO)
            SQG0(NO)=SQG0(NO)*SCL/SQCIB(NO)
            SQCIMX(NO)=MAX(SQCI(NO)+2.*SQF0(NO), 
     :                     ABS(SQCI(NO)-2.*SQG0(NO)))
         ELSE
            SCL=2.*VAR*CC(NO)
            SQCI(NO)=SQCI(NO)/SCL
            SQF0(NO)=SQF0(NO)/SCL
            SQG0(NO)=SQG0(NO)/SCL
            SQCIMX(NO)=MAX(SQCI(NO)+2.*SQF0(NO), 
     :                     ABS(SQCI(NO)-2.*SQG0(NO)))
         END IF
  310 CONTINUE
      WRITE(IOUT,740)
      CALL PRTOTB12(SQCI,GNAM,NOINT)
      WRITE(IOUT,750)
      CALL PRTOTB12(SQF0,GNAM,NOINT)
      WRITE(IOUT,760)
      CALL PRTOTB12(SQG0,GNAM,NOINT)
      WRITE(IOUT,770)
      CALL PRTOTB12(SQCIMX,GNAM,NOINT)
      STOP
C
  890 WRITE (IOUT,636)
C
  999 STOP
      END
C=======================================================================
      SUBROUTINE PRTOTB4(VAL,VID,NO)
C**PRINT VALUES IN THREE GROUPS OF THREE COLUMNS
      INTEGER IIN1, IOUT, K, L, NO, NR
      REAL VAL
      CHARACTER*4 VID(NO)
      DIMENSION VAL(NO)
      COMMON /ITP   / IIN1, IOUT
      NR = NO/3
      IF (3*NR.NE.NO) NR = NR + 1
      DO 10 K = 1, NR
        WRITE (IOUT,500) (L,VID(L),VAL(L),L=K,NO,NR)
   10 CONTINUE
      RETURN
  500 FORMAT (3X,3(I3,2X,A4,1X,G11.5,3X))
      END
C=======================================================================
      SUBROUTINE PRTOTB10(VAL,VID,NO)
C**PRINT VALUES IN TWO GROUPS OF THREE COLUMNS
      INTEGER IIN1, IOUT, K, L, NO, NR
      REAL VAL
      CHARACTER*10 VID(NO)
      DIMENSION VAL(NO)
      COMMON /ITP   / IIN1, IOUT
      NR = NO/2
      IF (2*NR.NE.NO) NR = NR + 1
      DO 10 K = 1, NR
        WRITE (IOUT,500) (L,VID(L),VAL(L),L=K,NO,NR)
   10 CONTINUE
      RETURN
  500 FORMAT (3X,2(I3,2X,A,1X,G11.5,3X))
      END
C=======================================================================
      SUBROUTINE PRTOTB12(VAL,VID,NO)
C**PRINT VALUES IN TWO GROUPS OF THREE COLUMNS
      INTEGER IIN1, IOUT, K, L, NO, NR
      REAL VAL
      CHARACTER*12 VID(NO)
      DIMENSION VAL(NO)
      COMMON /ITP   / IIN1, IOUT
      NR = NO/2
      IF (2*NR.NE.NO) NR = NR + 1
      DO 10 K = 1, NR
        WRITE (IOUT,500) (L,VID(L),VAL(L),L=K,NO,NR)
   10 CONTINUE
      RETURN
  500 FORMAT (3X,2(I3,2X,A,1X,G11.5,3X))
      END
C=======================================================================
      SUBROUTINE PRTOTD(W,NR,NC,NRD,OBSNAM,NH)
C     PRINT A FULL WEIGHT MATRIX FOR OBSERVATIONS DIVIDED VERTICALLY
C     INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NH, NR, NRD
      REAL W(NRD,NC)
      CHARACTER*12 OBSNAM(NC+NH)
      COMMON /ITP   / IIN1, IOUT
C
  500 FORMAT (/,1X,'OBSERVATION',2X,5(1X,A12))
  505 FORMAT (1X,A12,1X,5(1X,G12.5))
  510 FORMAT (1X,12('-'),1X,5(1X,A12))
C
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (OBSNAM(J),J=K+NH,J10+NH)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) OBSNAM(I+NH), (W(I,J),J=K,J10)
   10   CONTINUE
   20 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE PRTOTP(W,NR,NC,NRD,PARNAM,NIPR,IPD,NPD)
C     PRINT A FULL WEIGHT MATRIX FOR PRIOR INFORMATION DIVIDED
C     VERTICALLY INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NR, NRD, IPD, NPD
      INTEGER NIPR(IPD)
      REAL W(NRD,NC)
      CHARACTER*10 PARNAM(NPD)
      COMMON /ITP   / IIN1, IOUT
C
  500 FORMAT (/,1X,'PARAMETER',2X,5(2X,A10,1X))
  505 FORMAT (1X,A10,1X,5(1X,G12.5))
  510 FORMAT (1X,10('-'),1X,5(1X,A12))
C
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (PARNAM(NIPR(J)),J=K,J10)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) PARNAM(NIPR(I)), (W(I,J),J=K,J10)
   10   CONTINUE
   20 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE PRTOTX(X,NR,NC,NRD,OBSNAM,PARNAM)
C**PRINT MATRICES DIVIDED VERTICALLY INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NR, NRD
      REAL X(NRD,NC)
      CHARACTER*10 PARNAM(NR)
      CHARACTER*12 OBSNAM(NC)
      COMMON /ITP   / IIN1, IOUT
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (OBSNAM(J),J=K,J10)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) PARNAM(I), (X(I,J),J=K,J10)
   10   CONTINUE
   20 CONTINUE
  500 FORMAT (/,1X,'PARAMETER',3X,5(A12,1X))
  505 FORMAT (1X,A10,2X,5(G12.5,1X))
  510 FORMAT (1X,10('-'),2X,5(A12,1X))
      RETURN
      END
C=======================================================================
      INTEGER FUNCTION NONB_LEN(CHARVAR,LENGTH)
C
C FUNCTION TO RETURN NON-BLANK LENGTH OF CONTENTS OF A CHARACTER VARIABLE
C
C Variable list:
C   CHARVAR  = CHARACTER VARIABLE OF INTEREST
C   LENGTH   = DIMENSIONED LENGTH OF CHARVAR
C
      CHARACTER*(*) CHARVAR,C*1
      INTEGER LENGTH, K
C
      C = ' '
      K = LENGTH+1
C
      DO 20 WHILE (C .EQ. ' ' .AND. K .GE. 1)
        K = K-1
        C = CHARVAR(K:K)
 20   CONTINUE
C
      NONB_LEN = K
      RETURN
      END
C=======================================================================
      SUBROUTINE FINDINP(IOUT,OUTNAM,INWR)
C
      CHARACTER*80 NAMFIL, OBFNAM, OUTNAM, OUTNAMU
      CHARACTER*200 LINE
      LOGICAL EXISTS
      INTEGER INWR, IOUT, ISTAT, NC, N, ISTART, ISTOP, IU, NLEN,
     &        NONB_LEN, LLOC
      REAL R
C
C     NAMFIL is the name of the name file
C     OBFNAM is the name of the OBS file
C     OUTNAM is the name of the output filename base listed in
C            the OBS file
C     OUTNAMU is an upper-case version of OUTNAM, for comparison
C            with 'NONE'
C
  500 FORMAT(' Enter the name of the NAME file:',/)
  510 FORMAT(' The NAME file is: ',A)
  520 FORMAT(A)
  530 FORMAT(/,' Error reading NAME file.  Enter blank line to quit.',/)
  540 FORMAT(/,' Error opening NAME file.  Enter blank line to quit.',/)
  550 FORMAT(/,' Error opening file "',A,'"')
C
    1 CONTINUE
      NAMFIL = ' '
      ISTAT = 0
C *** The following statement should be uncommented in order to use
C *** GETARG to retrieve a command line argument.
      CALL GETARG(1,NAMFIL)
      IF(NAMFIL.NE.' ') THEN
        INQUIRE (FILE=NAMFIL,EXIST=EXISTS)
        IF(.NOT.EXISTS) THEN
          NC=INDEX(NAMFIL,' ')
          NAMFIL(NC:NC+3)='.nam'
          INQUIRE (FILE=NAMFIL,EXIST=EXISTS)
          IF(.NOT.EXISTS) THEN
            WRITE(*,*) ' Name file does not exist'
            STOP
          ENDIF
        ENDIF
      ELSE
        WRITE(*,500)
        READ(*,520,ERR=5,IOSTAT=ISTAT) NAMFIL
      ENDIF
      IF (NAMFIL.EQ.' ') STOP
    5 CONTINUE
      IF (ISTAT.NE.0) THEN
        WRITE(*,530)
        GOTO 1
      ENDIF
      OPEN(INWR,FILE=NAMFIL,STATUS='OLD',ERR=7,IOSTAT=ISTAT)
    7 CONTINUE
      IF (ISTAT.NE.0) THEN
        WRITE(*,540)
        GOTO 1
      ENDIF
      WRITE(*,510) NAMFIL
C
      OUTNAM=' '
   10 CONTINUE
C       Read down in NAME file to find name of OBS file
        CALL URDCOM(INWR,IOUT,LINE,ISTAT)
        IF (ISTAT.EQ.1) THEN
C         END-OF-FILE reached
          OUTNAM='NONE'
          GOTO 40
        ENDIF
        LLOC = 1
C       Look for OBS file type in the name file
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,1,N,R,IOUT,INWR)
        IF (LINE(ISTART:ISTOP).EQ.'OBS') THEN
C         Assign OBFNAM as the name of the OBS file, read from the NAME
C         file
          CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,IU,R,IOUT,INWR)
          CALL URWORD(LINE,LLOC,ISTART,ISTOP,0,N,R,IOUT,INWR)
          OBFNAM = LINE(ISTART:ISTOP)
          CLOSE(INWR)
          OPEN(INWR,FILE=OBFNAM,STATUS='OLD',ERR=20,IOSTAT=ISTAT)
   20     CONTINUE
          IF (ISTAT.NE.0) THEN
            NLEN = NONB_LEN(OBFNAM,80)
            WRITE(*,550) OBFNAM(1:NLEN)
            STOP
          ENDIF
C         Assign OUTNAM as filename base read from OBS file
          CALL URDCOM(INWR,IOUT,LINE,ISTAT)
          LLOC = 1
          CALL URWORD(LINE,LLOC,ISTART,ISTOP,0,N,R,IOUT,INWR)
          OUTNAM = LINE(ISTART:ISTOP)
          CLOSE(INWR)
          CALL UCASE(OUTNAM,OUTNAMU,1)
          IF (OUTNAMU.EQ.'NONE') OUTNAM = 'NONE'
          GOTO 40
        ENDIF
      GOTO 10
C
   40 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE URDCOM(IN,IOUT,LINE,ISTAT)
C
C-----VERSION 02FEB1999 URDCOM
C     ******************************************************************
C     READ COMMENTS FROM A FILE AND PRINT THEM.  RETURN THE FIRST LINE
C     THAT IS NOT A COMMENT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*(*) LINE
      INTEGER IN, IOUT, ISTAT, L, I
C     ------------------------------------------------------------------
      ISTAT=0
   10 READ(IN,'(A)',END=40,ERR=50) LINE
      IF(LINE(1:1).NE.'#') RETURN
      L=LEN(LINE)
      IF(L.GT.79) L=79
      DO 20 I=L,1,-1
      IF(LINE(I:I).NE.' ') GO TO 30
   20 CONTINUE
   30 WRITE(IOUT,'(1X,A)') LINE(1:I)
      GO TO 10
C
C     FLAG END OF FILE
   40 CONTINUE
      ISTAT=1
      RETURN
C
C     FLAG READ ERROR
   50 CONTINUE
      ISTAT=2
      RETURN
      END
C=======================================================================
      SUBROUTINE URWORD(LINE,ICOL,ISTART,ISTOP,NCODE,N,R,IOUT,IN)
C
C
C-----VERSION 1003 05AUG1992 URWORD
C     ******************************************************************
C     ROUTINE TO EXTRACT A WORD FROM A LINE OF TEXT, AND OPTIONALLY
C     CONVERT THE WORD TO A NUMBER.
C        ISTART AND ISTOP WILL BE RETURNED WITH THE STARTING AND
C          ENDING CHARACTER POSITIONS OF THE WORD.
C        THE LAST CHARACTER IN THE LINE IS SET TO BLANK SO THAT IF ANY
C          PROBLEMS OCCUR WITH FINDING A WORD, ISTART AND ISTOP WILL
C          POINT TO THIS BLANK CHARACTER.  THUS, A WORD WILL ALWAYS BE
C          RETURNED UNLESS THERE IS A NUMERIC CONVERSION ERROR.  BE SURE
C          THAT THE LAST CHARACTER IN LINE IS NOT AN IMPORTANT CHARACTER
C          BECAUSE IT WILL ALWAYS BE SET TO BLANK.
C        A WORD STARTS WITH THE FIRST CHARACTER THAT IS NOT A SPACE OR
C          COMMA, AND ENDS WHEN A SUBSEQUENT CHARACTER THAT IS A SPACE
C          OR COMMA.  NOTE THAT THESE PARSING RULES DO NOT TREAT TWO
C          COMMAS SEPARATED BY ONE OR MORE SPACES AS A NULL WORD.
C        FOR A WORD THAT BEGINS WITH "'", THE WORD STARTS WITH THE
C          CHARACTER AFTER THE QUOTE AND ENDS WITH THE CHARACTER
C          PRECEDING A SUBSEQUENT QUOTE.  THUS, A QUOTED WORD CAN
C          INCLUDE SPACES AND COMMAS.  THE QUOTED WORD CANNOT CONTAIN
C          A QUOTE CHARACTER.
C        IF NCODE IS 1, THE WORD IS CONVERTED TO UPPER CASE.
C        IF NCODE IS 2, THE WORD IS CONVERTED TO AN INTEGER.
C        IF NCODE IS 3, THE WORD IS CONVERTED TO A REAL NUMBER.
C        NUMBER CONVERSION ERROR IS WRITTEN TO UNIT IOUT IF IOUT IS
C          POSITIVE; ERROR IS WRITTEN TO DEFAULT OUTPUT IF IOUT IS 0;
C          NO ERROR MESSAGE IS WRITTEN IF IOUT IS NEGATIVE.
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*(*) LINE
      CHARACTER*20 RW,STRING
C     ------------------------------------------------------------------
C
C1------Set last char in LINE to blank and set ISTART and ISTOP to point
C1------to this blank as a default situation when no word is found.  If
C1------starting location in LINE is out of bounds, do not look for a
C1------word.
      LINLEN=LEN(LINE)
      LINE(LINLEN:LINLEN)=' '
      ISTART=LINLEN
      ISTOP=LINLEN
      LINLEN=LINLEN-1
      IF(ICOL.LT.1 .OR. ICOL.GT.LINLEN) GO TO 100
C
C2------Find start of word, which is indicated by first character that
C2------is not a blank and not a comma.
      DO 10 I=ICOL,LINLEN
      IF(LINE(I:I).NE.' ' .AND. LINE(I:I).NE.',') GO TO 20
10    CONTINUE
      ICOL=LINLEN+1
      GO TO 100
C
C3------Found start of word.  Look for end.
C3A-----When word is quoted, only a quote can terminate it.
20    IF(LINE(I:I).EQ.'''') THEN
         I=I+1
         IF(I.LE.LINLEN) THEN
            DO 25 J=I,LINLEN
            IF(LINE(J:J).EQ.'''') GO TO 40
25          CONTINUE
         END IF
C
C3B-----When word is not quoted, space or comma will terminate.
      ELSE
         DO 30 J=I,LINLEN
         IF(LINE(J:J).EQ.' ' .OR. LINE(J:J).EQ.',') GO TO 40
30       CONTINUE
      END IF
C
C3C-----End of line without finding end of word; set end of word to
C3C-----end of line.
      J=LINLEN+1
C
C4------Found end of word; set J to point to last character in WORD and
C-------set ICOL to point to location for scanning for another word.
40    ICOL=J+1
      J=J-1
      IF(J.LT.I) GO TO 100
      ISTART=I
      ISTOP=J
C
C5------Convert word to upper case and RETURN if NCODE is 1.
      IF(NCODE.EQ.1) THEN
         IDIFF=ICHAR('a')-ICHAR('A')
         DO 50 K=ISTART,ISTOP
            IF(LINE(K:K).GE.'a' .AND. LINE(K:K).LE.'z')
     1             LINE(K:K)=CHAR(ICHAR(LINE(K:K))-IDIFF)
50       CONTINUE
         RETURN
      END IF
C
C6------Convert word to a number if requested.
100   IF(NCODE.EQ.2 .OR. NCODE.EQ.3) THEN
         RW=' '
         L=20-ISTOP+ISTART
         IF(L.LT.1) GO TO 200
         RW(L:20)=LINE(ISTART:ISTOP)
         IF(NCODE.EQ.2) READ(RW,'(I20)',ERR=200) N
         IF(NCODE.EQ.3) READ(RW,'(F20.0)',ERR=200) R
      END IF
      RETURN
C
C7------Number conversion error.
200   IF(NCODE.EQ.3) THEN
         STRING= 'A REAL NUMBER'
         L=13
      ELSE
         STRING= 'AN INTEGER'
         L=10
      END IF
C
C7A-----If output unit is negative, set last character of string to 'E'.
      IF(IOUT.LT.0) THEN
         N=0
         R=0.
         LINE(LINLEN+1:LINLEN+1)='E'
         RETURN
C
C7B-----If output unit is positive; write a message to output unit.
      ELSE IF(IOUT.GT.0) THEN
         IF(IN.GT.0) THEN
            WRITE(IOUT,201) IN,LINE(ISTART:ISTOP),STRING(1:L),LINE
         ELSE
            WRITE(IOUT,202) LINE(ISTART:ISTOP),STRING(1:L),LINE
         END IF
201      FORMAT(1X,/1X,'FILE UNIT',I4,' : ERROR CONVERTING "',A,
     1       '" TO ',A,' IN LINE:',/1X,A)
202      FORMAT(1X,/1X,'KEYBOARD INPUT : ERROR CONVERTING "',A,
     1       '" TO ',A,' IN LINE:',/1X,A)
C
C7C-----If output unit is 0; write a message to default output.
      ELSE
         IF(IN.GT.0) THEN
            WRITE(*,201) IN,LINE(ISTART:ISTOP),STRING(1:L),LINE
         ELSE
            WRITE(*,202) LINE(ISTART:ISTOP),STRING(1:L),LINE
         END IF
      END IF
C
C7D-----STOP after writing message.
      STOP
      END
C=======================================================================
      SUBROUTINE UCASE(WORDIN,WORDOUT,ICASE)
C     VERSION 19980316 ERB
C     ******************************************************************
C     CONVERT A CHARACTER STRING TO ALL UPPER (ICASE > 0) OR ALL
C     LOWER (ICASE < 0) CASE.  IF ICASE = 0, NO CASE CONVERSION IS DONE.
C     ******************************************************************
C       SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER WORDIN*(*), WORDOUT*(*)
      INTEGER ICASE, LENGNB, LENGIN, LENGOUT
C
C     DETERMINE LENGTH OF STRING VARIABLES
      LENGIN = LEN(WORDIN)
      LENGOUT = LEN(WORDOUT)
C
C     DETERMINE IF WORDOUT IS LONG ENOUGH TO CONTAIN NON-BLANK LENGTH
C     OF WORDIN
      LENGNB = NONB_LEN(WORDIN,LENGIN)
      IF (LENGNB.GT.LENGOUT) STOP 'STRING-LENGTH ERROR IN UCASE'
C
      WORDOUT = WORDIN
      IDIFF=ICHAR('a')-ICHAR('A')
      IF (ICASE.GT.0) THEN
C       CONVERT STRING TO UPPER CASE
        DO 10 K=1,LENGNB
          IF(WORDIN(K:K).GE.'a' .AND. WORDIN(K:K).LE.'z')
     1      WORDOUT(K:K)=CHAR(ICHAR(WORDIN(K:K))-IDIFF)
10      CONTINUE
C
      ELSEIF (ICASE.LT.0) THEN
C       CONVERT STRING TO LOWER CASE
        DO 20 K=1,LENGNB
          IF(WORDIN(K:K).GE.'A' .AND. WORDIN(K:K).LE.'Z')
     1      WORDOUT(K:K)=CHAR(ICHAR(WORDIN(K:K))+IDIFF)
20      CONTINUE
C
      ENDIF
C
      RETURN
      END
C=======================================================================
      SUBROUTINE SOLVE(C,D,G,NPE,NPD,IND,ISTOP,DET)
C-----VERSION 1001 25FEB2003 (SC)
C     ******************************************************************
C         SOLVE SYSTEM OF LINEAR EQUATIONS BY LU-DECOMPOSITION
C     ******************************************************************
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      DOUBLE PRECISION C(NPD,NPD),D(NPD),G(NPD),DPIV,DTMPA,DSUM
      REAL DET
      INTEGER ISTOP,IND,NPE,NM1,K,KP1,J,I,IP1,NPD
C     ------------------------------------------------------------------
C         COMPUTE TRIAL PARAMETER STEP LENGTHS USING LDU FACTORIZATION:
C           DECOMPOSE MATRIX
      NM1=NPE-1
      ISTOP=0
C
C-----------LDU FACTORIZATION
      IF (IND.GT.0) THEN
         DET=1.
         DO 30 K=1,NM1
            DPIV=C(K,K)
            DET=DET*DPIV
            IF (DPIV.LT.1.D-13) THEN
               ISTOP=1
               RETURN
            ENDIF
            DPIV=1.D00/DPIV
            KP1=K+1
            DO 20 J=KP1,NPE
               DTMPA=C(J,K)*DPIV
               DO 10 I=J,NPE
                  C(I,J)=C(I,J)-DTMPA*C(I,K)
   10          CONTINUE
   20       CONTINUE
            C(K,K)=DPIV
   30    CONTINUE
         DET=DET*C(NPE,NPE)
         IF(C(NPE,NPE).LT.1.D-13) THEN
            ISTOP=1
            RETURN
         ENDIF
      ENDIF
C
C-----------INITIALIZE D
      DO 40 K=1,NPE
         D(K)=G(K)
   40 CONTINUE
C
C-----------FORWARD SUBSTITUTE
      DO 60 K=1,NM1
         DTMPA=D(K)*C(K,K)
         KP1=K+1
         DO 50 J=KP1,NPE
            D(J)=D(J)-C(J,K)*DTMPA
   50    CONTINUE
   60 CONTINUE
C
C-----------BACKWARD SUBSTITUTE
      D(NPE)=D(NPE)/C(NPE,NPE)
      DO 90 I=NPE-1,1,-1
         IP1=I+1
         DSUM=0.
         DO 70 J=IP1,NPE
            DSUM=DSUM+C(J,I)*D(J)
   70    CONTINUE
         D(I)=(D(I)-DSUM)*C(I,I)
   90 CONTINUE
C
      RETURN
      END
C=======================================================================
      SUBROUTINE SETC(NPE,NOBS,NH,NQ,MPR,IPR,C,G,X,WT,WQ,PRM,WP,
     &                WTP,SCLE,NIPR,NPD)
C
C    COMPUTE AND FACTOR C (THE COEFFICIENT MATRIX)
C
      INTEGER NPE,NOBS,NH,NQ,MPR,IPR,NIPR(IPR),NPD
      DOUBLE PRECISION C(NPE,NPE),G(NPE),SCLE(NPE)
      REAL X(NPE,NOBS),WT(NH),WQ(NQ,NQ),WP(MPR),WTP(IPR,IPR),
     &     PRM(NPE+1,MPR)
      INTEGER IP,I,J,N,K,K1,L,L1,ISTOP,IMP
      DOUBLE PRECISION DTMPA
      REAL W,DET
C
C----------INITIALIZE C
      DO 20 IP=1,NPE
         DO 10 I=1,NPE
            C(I,IP)=0.D00
   10    CONTINUE
         G(IP)=0.
   20 CONTINUE
C
C----------CALCULATE SENSITIVITY CONTRIBUTIONS TO C
C
C---- HEADS
      IF (NH .GT. 0) THEN
         DO 50 N=1,NH
            W=WT(N)
            IF(W.LT.0.) W=1.E-20
            DO 40 IP=1,NPE
               DTMPA=DBLE(W)*DBLE(X(IP,N))
               DO 30 I=IP,NPE
                  C(I,IP)=DBLE(X(I,N))*DTMPA+C(I,IP)
   30          CONTINUE
   40       CONTINUE
   50    CONTINUE
      ENDIF
C
C-----FLOWS
      IF (NQ .GT. 0) THEN
        DO 100 J = 1, NPE
          DO 90 K = 1, NQ
            DTMPA = 0.0
            IF (WQ(K,K).GT.0.0) THEN
              DO 70 L = 1, NQ
                IF (WQ(L,L).GT.0.0) THEN
                  DTMPA = DTMPA + DBLE(WQ(K,L))*DBLE(X(J,NH+L))
                ENDIF
   70         CONTINUE
            ENDIF
            DO 80 I = J, NPE
              C(I,J) = C(I,J) + DBLE(X(I,NH+K))*DTMPA
   80       CONTINUE
   90     CONTINUE
  100   CONTINUE
      ENDIF
C
C-----PRIOR INFORMATION ON PARAMETER SUMS
      IF(MPR.GT.0) THEN
         DO 130 IMP=1,MPR
            DO 120 IP=1,NPE
               IF(PRM(IP,IMP).NE.0.) THEN
                  DTMPA=DBLE(WP(IMP))*DBLE(PRM(IP,IMP))
                  DO 110 I=IP,NPE
                     C(I,IP)=DBLE(PRM(I,IMP))*DTMPA+C(I,IP)
  110             CONTINUE
               ENDIF
  120       CONTINUE
  130    CONTINUE
      ENDIF
C
C-----CORRELATED PRIOR
      IF (IPR.GT.0) THEN
         DO 170 I = 1, NPE
           DO 160 J = I, NPE
             DO 150 K = 1, IPR
               K1 = NIPR(K)
               IF (K1.EQ.I) THEN
                  DO 140 L = 1, IPR
                     W = WTP(L,K)
                     L1 = NIPR(L)
                     IF (L1.EQ.J) C(J,I) = C(J,I) + W
  140             CONTINUE
               ENDIF
  150        CONTINUE
  160      CONTINUE
  170    CONTINUE
      ENDIF
C
C-------SCALE COEFFICIENT MATRIX AND GRADIENT VECTOR
      DO 210 IP=1,NPE
         SCLE(IP)=1.D00
         IF(C(IP,IP).GT.1.D-30) SCLE(IP)=DSQRT(C(IP,IP))
  210 CONTINUE
      IF(NPE.GT.1) THEN
         DO 230 IP=1,NPE-1
            DTMPA=SCLE(IP)
            DO 220 I=IP+1,NPE
               C(I,IP)=C(I,IP)/(SCLE(I)*DTMPA)
               C(IP,I)=C(I,IP)
  220       CONTINUE
            C(IP,IP)=1.D00
  230    CONTINUE
      ENDIF
      C(NPE,NPE)=1.D00
C
C-------COMPUTE SCALED DISPLACEMENT COMPONENTS
      CALL SOLVE(C,G,G,NPE,NPD,1,ISTOP,DET)
      
C
      END
C=======================================================================
      SUBROUTINE SETG(NPE,NOBS,NH,NQ,C,G,X,WT,WQ,SCLE,F,FOPT,B,BOPT,NPD,
     &                IDG,DG,VYP,GP,GOPT,GPD,D,Z)
C
C     COMPUTE G (THE RHS VECTOR) AND SOLVE FOR PARAMETER CORRECTIONS
C     NOTE THAT ONLY OBSERVATIONS OF MODEL FUNCTIONS (HEADS AND FLOWS)
C      CONTRIBUTE TO G=X'*WGT*(F-(FOPT+X*(B-BOPT))). EXTRA TERMS ARE
C      COMPUTED AND ADDED FOR CONFIDENCE OR PREDICTION INTERVAL WHEN
C      IDG .NE. 0.
C     SOLVING MODIFIES G SO AS OUTPUT G=C*G, I.E.
C      G=(X'*WGT*X)^(-1)*X'*WGT*(F-FOPT-X*(B-BOPT)).
C     ALSO ON OUTPUT, F=F-FOPT-X*(B-BOPT).
C
      INTEGER NPE,NOBS,NH,NQ,NPD,IDG
      DOUBLE PRECISION C(NPE,NPE),G(NPE),SCLE(NPE),D(NPE)
      REAL X(NPE,NOBS),WT(NH),WQ(NQ,NQ),F(NOBS),FOPT(NOBS),B(NPE),
     &     BOPT(NPE),GP,GOPT,GPD,DG,VYP,Z(NPE)
      INTEGER IP,I,N,K,L,ISTOP
      DOUBLE PRECISION TMP
      REAL W,DET
C
C----------INITIALIZE G AND COMPUTE F=F-(FOPT+X*(B-BOPT))
      DO 10 IP=1,NPE
         G(IP)=0.
   10 CONTINUE
      DO 30 N=1,NOBS
         F(N)=F(N)-FOPT(N)
         DO 20 IP=1,NPE
            F(N)=F(N)-X(IP,N)*(B(IP)-BOPT(IP))
   20    CONTINUE
   30 CONTINUE
C
C----------CALCULATE SENSITIVITY CONTRIBUTIONS TO G.
C
C---- HEADS
      IF (NH .GT. 0) THEN
         DO 50 N=1,NH
            W=WT(N)
            IF(W.LT.0.) W=1.E-20
            DO 40 IP=1,NPE
               G(IP)=G(IP) + DBLE(W)*DBLE(X(IP,N))*DBLE(F(N))
   40       CONTINUE
   50    CONTINUE
      ENDIF
C
C-----FLOWS
      IF (NQ .GT. 0) THEN
        DO 90 I = 1, NPE
          DO 80 K = 1, NQ
            TMP = 0.0
            IF (WQ(K,K).GT.0.0) THEN
              DO 70 L = 1, NQ
                IF (WQ(L,L).GT.0.0) TMP=TMP+DBLE(WQ(K,L))*DBLE(F(NH+L))
   70         CONTINUE
            ENDIF
            G(I) = G(I) + DBLE(X(I,NH+K))*TMP
   80     CONTINUE
   90   CONTINUE
      ENDIF
C
C     COMPUTE AND ADD EXTRA TERMS FOR CONFIDENCE OR PREDICTION INTERVALS
      IF (IDG.NE.0) THEN
         GPD=GP-GOPT
         DG=0.
         DO 100 IP=1,NPE
            GPD=GPD-Z(IP)*(B(IP)-BOPT(IP))
            DG=D(IP)*G(IP)
  100    CONTINUE
         DG=DG/VYP
         DO 110 IP=1,NPE
            G(IP)=G(IP)-DG*Z(IP)
  110    CONTINUE
      END IF   
C      
C----------SOLVE FOR PARAMETER CORRECTIONS
      DO 120 IP=1,NPE
         G(IP)=G(IP)/SCLE(IP)
  120 CONTINUE
      CALL SOLVE(C,G,G,NPE,NPD,0,ISTOP,DET)
      DO 130 IP=1,NPE
         G(IP)=G(IP)/SCLE(IP)
  130 CONTINUE
C
      RETURN
      END
C=======================================================================
      REAL FUNCTION SUMCI(NPE,NOBS,NH,NQ,MPR,IPR,F,WT,WQ,X,G,PRM,WP,WTP,
     &                    NIPR,D)
C
C     COMPUTE CI=D*X'*WGT*(F-X*PSI) WHERE D=Z*(X'*WGT*X)^-1
C
      INTEGER NPE,NOBS,NH,NQ,N,K,L,IP,MPR,IPR,NIPR(IPR)
      DOUBLE PRECISION D(NPE),G(NPE)
      REAL F(NOBS),WT(NH),WQ(NQ,NQ),W,X(NPE,NOBS),FD,
     &     PRM(NPE+1,MPR),WP(MPR),WTP(IPR,IPR),XD
C
      SUMCI=0.
C      
C-----HEADS
      IF (NH.GT.0) THEN
         DO 20 N=1,NH
            W=WT(N)
            IF (W.GT.0.) THEN
               FD=0.
               XD=0.
               DO 10 IP=1,NPE
                  XD=XD+X(IP,N)*D(IP)
                  FD=FD+X(IP,N)*G(IP)
   10          CONTINUE
               FD=W*(F(N)-FD)
               SUMCI=SUMCI+XD*FD
            END IF
   20    CONTINUE           
      END IF   
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 60 K=1,NQ
            IF (WQ(K,K).GT.0.) THEN
               W=0.0
               DO 40 L=1,NQ
                  IF (WQ(L,L).GT.0.) THEN
                     FD=0.
                     N=NH+L
                     DO 30 IP=1,NPE
                        FD=FD+X(IP,N)*G(IP)
   30                CONTINUE
                     FD=F(N)-FD
                     W=W+WQ(K,L)*FD
                  END IF
   40          CONTINUE
               XD=0.
               N=NH+K
               DO 50 IP=1,NPE
                  XD=XD+X(IP,N)*D(IP)
   50          CONTINUE
               SUMCI=SUMCI+W*XD
            END IF
   60    CONTINUE
      END IF
C      
C-----PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
         DO 140 N=1,MPR
            FD= 0.0
            XD=0.
            DO 130 IP=1,NPE
               FD=FD+PRM(IP,N)*G(IP)
               XD=XD+PRM(IP,N)*D(IP)
  130       CONTINUE
            SUMCI=SUMCI-XD*WP(N)*FD
  140    CONTINUE
      END IF
C
C-----PRIOR WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
         DO 160 K=1,IPR
            FD=0.
            DO 150 L=1,IPR
               FD=FD+WTP(K,L)*G(NIPR(L))
  150       CONTINUE
            SUMCI=SUMCI-FD*D(NIPR(K))
  160    CONTINUE
      END IF
      
C
      END
C=======================================================================
      REAL FUNCTION SUMCIB(NPE,NOBS,NH,NQ,MPR,IPR,WT,WQ,X,G,PRM,WP,WTP,
     &                    NIPR,D,B,BOPT)
C
C     COMPUTE CIB=D*X'*WGT*(FOPT-FOPT-X*PSI) WHERE D=Z*(X'*WGT*X)^-1
C
      INTEGER NPE,NOBS,NH,NQ,N,K,L,IP,MPR,IPR,NIPR(IPR)
      DOUBLE PRECISION D(NPE),G(NPE)
      REAL WT(NH),WQ(NQ,NQ),W,X(NPE,NOBS),FD,
     &     PRM(NPE+1,MPR),WP(MPR),WTP(IPR,IPR),XD,B(NPE),BOPT(NPE)
C
      SUMCIB=0.
C      
C-----HEADS
      IF (NH.GT.0) THEN
         DO 20 N=1,NH
            W=WT(N)
            IF (W.GT.0.) THEN
               FD=0.
               XD=0.
               DO 10 IP=1,NPE
                  XD=XD+X(IP,N)*D(IP)
                  FD=FD+X(IP,N)*(G(IP)+B(IP)-BOPT(IP))
   10          CONTINUE
               SUMCIB=SUMCIB-XD*W*FD
            END IF
   20    CONTINUE           
      END IF   
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 60 K=1,NQ
            IF (WQ(K,K).GT.0.) THEN
               W=0.0
               DO 40 L=1,NQ
                  IF (WQ(L,L).GT.0.) THEN
                     FD=0.
                     N=NH+L
                     DO 30 IP=1,NPE
                        FD=FD+X(IP,N)*(G(IP)+B(IP)-BOPT(IP))
   30                CONTINUE
                     W=W+WQ(K,L)*FD
                  END IF
   40          CONTINUE
               XD=0.
               N=NH+K
               DO 50 IP=1,NPE
                  XD=XD+X(IP,N)*D(IP)
   50          CONTINUE
               SUMCIB=SUMCIB-W*XD
            END IF
   60    CONTINUE
      END IF
C      
C-----PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
         DO 140 N=1,MPR
            FD= 0.0
            XD=0.
            DO 130 IP=1,NPE
               FD=FD+PRM(IP,N)*(G(IP)+B(IP)-BOPT(IP))
               XD=XD+PRM(IP,N)*D(IP)
  130       CONTINUE
CC IS SIGN WRONG SO THIS CONTRIBUTION SHOULD BE ADDED INSTEAD OF SUBTRACTED?
            SUMCIB=SUMCIB-XD*WP(N)*FD
  140    CONTINUE
      END IF
C
C-----PRIOR WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
         DO 160 K=1,IPR
            FD=0.
            DO 150 L=1,IPR
               IP=NIPR(L)
               FD=FD+WTP(K,L)*(G(IP)+B(IP)-BOPT(IP))
  150       CONTINUE
CC IS SIGN WRONG SO THIS CONTRIBUTION SHOULD BE ADDED INSTEAD OF SUBTRACTED?
            SUMCIB=SUMCIB-FD*D(NIPR(K))
  160    CONTINUE
      END IF
      
C
      END
C=======================================================================
      REAL FUNCTION SUMQI(NPE,NOBS,NH,NQ,MPR,IPR,F,WT,WQ,X,G,PRM,WP,WTP,
     &                    NIPR)
C
C     COMPUTE QI=(F-FOPT-X*PSI)'*WGT*(F-X*PSI)+(XD*PSI)'*WGTD*(XD*PSI)
C       WHERE PSI=(X'*WGT*X)^(-1)*X'*WGT*(F-FOPT)=G+X*(B-BOPT); (PLUS SOME MORE FOR INTERVALS)
C     NOTE AS INPUT F HAS BEEN CHANGED SO F=F-FOPT-X*(B-BOPT), AND
C       G=(X'*WGT*X)^(-1)*X'*WGT*(F-FOPT-X*(B-BOPT)).
C
      INTEGER NPE,NOBS,NH,NQ,N,K,L,IP,MPR,IPR,NIPR(IPR)
      REAL F(NOBS),WT(NH),WQ(NQ,NQ),W,X(NPE,NOBS),FD,
     &     PRM(NPE+1,MPR),WP(MPR),WTP(IPR,IPR)
      DOUBLE PRECISION G(NPE)
C
      SUMQI=0.
C      
C-----HEADS
      IF (NH.GT.0) THEN
         DO 20 N=1,NH
            W=WT(N)
            IF (W.GT.0.) THEN
               FD=0.
               DO 10 IP=1,NPE
                  FD=FD+X(IP,N)*G(IP)
   10          CONTINUE
               FD=F(N)-FD
               SUMQI=SUMQI+FD*W*FD
            END IF
   20    CONTINUE           
      END IF   
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 60 K=1,NQ
            IF (WQ(K,K).GT.0.) THEN
               W=0.0
               DO 40 L=1,NQ
                  IF (WQ(L,L).GT.0.) THEN
                     FD=0.
                     N=NH+L
                     DO 30 IP=1,NPE
                        FD=FD+X(IP,N)*G(IP)
   30                CONTINUE
                     FD=F(N)-FD
                     W=W+WQ(K,L)*FD
                  END IF
   40          CONTINUE
               FD=0.
               N=NH+K
               DO 50 IP=1,NPE
                  FD=FD+X(IP,N)*G(IP)
   50          CONTINUE
               FD=F(N)-FD
               SUMQI=SUMQI+W*FD
            END IF
   60    CONTINUE
      END IF
C      
C-----PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
         DO 140 N=1,MPR
            FD= 0.0
            DO 130 IP=1,NPE
               FD=FD+PRM(IP,N)*G(IP)
  130       CONTINUE
            SUMQI=SUMQI+FD*WP(N)*FD
  140    CONTINUE
      END IF
C
C-----PRIOR WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
         DO 160 K=1,IPR
            FD=0.
            DO 150 L=1,IPR
               FD=FD+WTP(K,L)*G(NIPR(L))
  150       CONTINUE
            SUMQI=SUMQI+FD*G(NIPR(K))
  160    CONTINUE
      END IF
      
C
      END
C=======================================================================
      REAL FUNCTION SUMQL(NPE,NOBS,NH,NQ,MPR,IPR,WT,WQ,X,PRM,WP,
     &                    WTP,NIPR,B,BOPT)
C
C     COMPUTE QL=F'*WGT*F WHERE F=FOPT-FOPT-X*(B-BOPT).
C
      INTEGER NPE,NOBS,NH,NQ,N,K,L,IP,MPR,IPR,NIPR(IPR)
      REAL WT(NH),WQ(NQ,NQ),W,X(NPE,NOBS),FL,PRM(NPE+1,MPR),
     &     WP(MPR),WTP(IPR,IPR),B(NPE),BOPT(NPE)
C
      SUMQL=0.
C
C-----HEADS
      IF (NH.GT.0) THEN
         DO 20 N=1,NH
            W=WT(N)
            IF (W.GT.0.) THEN
               FL=0.
               DO 10 IP=1,NPE
                  FL=FL+X(IP,N)*(B(IP)-BOPT(IP))
   10          CONTINUE
               SUMQL=SUMQL+FL*W*FL
            END IF
   20    CONTINUE           
      END IF   
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 60 K=1,NQ
            IF (WQ(K,K).GT.0.) THEN
               W=0.0
               DO 40 L=1,NQ
                  IF (WQ(L,L).GT.0.) THEN
                     N=NH+L
                     FL=0.
                     DO 30 IP=1,NPE
                        FL=FL+X(IP,N)*(B(IP)-BOPT(IP))
   30                CONTINUE
                     W=W+WQ(K,L)*FL
                  END IF
   40          CONTINUE
               N=NH+K
               FL=0.
               DO 50 IP=1,NPE
                  FL=FL+X(IP,N)*(B(IP)-BOPT(IP))
   50          CONTINUE
               SUMQL=SUMQL+W*FL
            END IF
   60    CONTINUE
      END IF
C      
C-----PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
         DO 140 N=1,MPR
            FL= 0.0
            DO 130 IP=1,NPE
               FL=FL+PRM(IP,N)*(B(IP)-BOPT(IP))
  130       CONTINUE
            SUMQL=SUMQL+FL*WP(N)*FL
  140    CONTINUE
      END IF
C
C-----PRIOR WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
         DO 160 K=1,IPR
            FL=0.
            DO 150 L=1,IPR
               IP=NIPR(L)
               FL=FL+WTP(K,L)*(B(IP)-BOPT(IP))
  150       CONTINUE
            IP=NIPR(K)
            SUMQL=SUMQL+FL*(B(IP)-BOPT(IP))
  160    CONTINUE
      END IF
      
C
      END
C=======================================================================
      REAL FUNCTION SUMQLB(NPE,NOBS,NH,NQ,MPR,IPR,WT,WQ,X,G,PRM,WP,
     &                     WTP,NIPR,B,BOPT)
C
C     COMPUTE QLB=(FOPT-FOPT-X*PSI)'*WGTM*(FOPT-FOPT-X*PSI)+(XD*PSI)'*WGTD*(XD*PSI)
C       WHERE PSI=(X'*WGT*X)^(-1)*X'*WGT*(F-FOPT)=G+X*(B-BOPT); (PLUS SOME MORE FOR INTERVALS)
C       NOTE G=(X'*WGT*X)^(-1)*X'*WGT*(F-FOPT-X*(B-BOPT)).
C
      INTEGER NPE,NOBS,NH,NQ,N,K,L,IP,MPR,IPR,NIPR(IPR)
      REAL WT(NH),WQ(NQ,NQ),W,X(NPE,NOBS),FD,
     &     PRM(NPE+1,MPR),WP(MPR),WTP(IPR,IPR),BOPT(NPE),B(NPE)
      DOUBLE PRECISION G(NPE)
C
      SUMQLB=0.
C      
C-----HEADS
      IF (NH.GT.0) THEN
         DO 20 N=1,NH
            W=WT(N)
            IF (W.GT.0.) THEN
               FD=0.
               DO 10 IP=1,NPE
                  FD=FD+X(IP,N)*(G(IP)+B(IP)-BOPT(IP))
   10          CONTINUE
               SUMQLB=SUMQLB+FD*W*FD
            END IF
   20    CONTINUE           
      END IF   
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 60 K=1,NQ
            IF (WQ(K,K).GT.0.) THEN
               W=0.0
               DO 40 L=1,NQ
                  IF (WQ(L,L).GT.0.) THEN
                     FD=0.
                     N=NH+L
                     DO 30 IP=1,NPE
                        FD=FD+X(IP,N)*(G(IP)+B(IP)-BOPT(IP))
   30                CONTINUE
                     W=W+WQ(K,L)*FD
                  END IF
   40          CONTINUE
               FD=0.
               N=NH+K
               DO 50 IP=1,NPE
                  FD=FD+X(IP,N)*(G(IP)+B(IP)-BOPT(IP))
   50          CONTINUE
               SUMQLB=SUMQLB+W*FD
            END IF
   60    CONTINUE
      END IF
C      
C-----PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
         DO 140 N=1,MPR
            FD= 0.0
            DO 130 IP=1,NPE
               FD=FD+PRM(IP,N)*(G(IP)+B(IP)-BOPT(IP))
  130       CONTINUE
            SUMQLB=SUMQLB+FD*WP(N)*FD
  140    CONTINUE
      END IF
C
C-----PRIOR WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
         DO 160 K=1,IPR
            FD=0.
            DO 150 L=1,IPR
               IP=NIPR(L)
               FD=FD+WTP(K,L)*(G(IP)+B(IP)-BOPT(IP))
  150       CONTINUE
            IP=NIPR(K)
            SUMQLB=SUMQLB+FD*(G(IP)+B(IP)-BOPT(IP))
  160    CONTINUE
      END IF
      
C
      END
C=======================================================================
      REAL FUNCTION SUMQT(NOBS,NH,NQ,F,WT,WQ)
C
C     COMPUTE QT=F'*WGT*F WHERE F=F-FOPT-X*(B-BOPT).
C
      INTEGER NOBS,NH,NQ,N,K,L
      REAL F(NOBS),WT(NH),WQ(NQ,NQ),W
C
      SUMQT=0.
C      
C-----HEADS
      IF (NH.GT.0) THEN
         DO 10 N=1,NH
            W=WT(N)
            IF(W.LT.0.) W=1.E-20
            SUMQT=SUMQT+F(N)*W*F(N)
   10    CONTINUE           
      END IF   
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 110 K=1,NQ
            IF (WQ(K,K).GT.0.) THEN
               W=0.0
               DO 100 L=1,NQ
                  IF (WQ(L,L).GT.0.) W=W+WQ(K,L)*F(NH+L)
  100          CONTINUE
               SUMQT=SUMQT+W*F(NH+K)
            END IF
  110    CONTINUE
      END IF   
C   
      END
