C     Last change:  SC   31 Oct 2003   12:50 pm
      PROGRAM CORFAC2k
C     ******************************************************************
C       CALCULATES CORRECTION FACTORS AND APPROXIMATE CORRECTION FACTORS
C        FOR CONFIDENCE REGION, CONFIDENCE AND PREDICTION INTERVALS
C
C       PROGRAM BY R. L. COOLEY, USGS, DENVER, COLO.
C       MODIFIED FOR MODFLOW-2000 BY STEEN CHRISTENSEN, DEPT. OF EARTH
C         SCIENCES, UNIVERSITY OF AARHUS, DENMARK. 
C       USES PIECES FROM BEALE2K BY E.R. BANTA 8/12/1999
C     ******************************************************************
C     SPECIFICATIONS:
C     ------------------------------------------------------------------
C-------ASSIGN VERSION NUMBER AND DATE
      CHARACTER*40 VERSION
      PARAMETER (VERSION='1.0.0 31/10/2003')
      DOUBLE PRECISION VAR
      DOUBLE PRECISION, ALLOCATABLE :: V(:,:), CF(:,:), D(:,:)
      CHARACTER*10, ALLOCATABLE :: PARNAM(:)
      CHARACTER*12, ALLOCATABLE :: OBSNAM(:), GNAM(:)
      REAL, ALLOCATABLE :: X(:,:), WH(:), WQ(:,:), WP(:), WG(:), Z(:,:),
     &                     WTP(:,:), Q(:), CP(:)
      INTEGER, ALLOCATABLE :: NIPR(:)     
      INTEGER I, IIN1, IMP, IOUT, IP, J, MPR, IPR, 
     &        NQ, NH, NOBS, NPD, IDIF, ITYP,NPI, NQI, NHI, NOINT,
     &        NPE, LENGNAM, NONB_LEN, IOS, NQD, IPD, ITRN, NTOT, 
     &        NPRI, KNT
      REAL VP, EC, TPCF, TOCF, CFSM, DF, A, B, COR, FAC, CCR, OMEC,
     &     DFB, FACB, CRB, VE, SUM, SSQP, QQUD, QSUM, QSMP, QSMN, SSQM,
     &     XI, CCI, XIPB, XINB, XIMX, CCIB, COVP, CPI, CPIB
      COMMON /ITP   / IIN1, IOUT
      CHARACTER*80 OUTNAM
      CHARACTER*84 FN
C     ------------------------------------------------------------------      
C
C**FORMAT LIST
  400 FORMAT(' ERROR OPENING FILE "',A,'" -- STOP EXECUTION')
  480 FORMAT (35X,'CORFAC-2k',//,
     &16X,'MODFLOW-2000 POST-PROCESSING PROGRAM TO CALCULATE',/,
     &1X,'CORRECTION FACTORS FOR CONFIDENCE REGION AND CONFIDE',
     &    'NCE AND PREDICTION INTERVALS',/,
     &29X,'Version ',A/)
  490 FORMAT(
     &' ERROR FINDING INPUT FILES.  OUTNAM MUST BE SPECIFIED AS A',/,
     &' STRING OTHER THAN "NONE" IN THE OBS FILE.  THE FILE',/,
     &' CORFAC.ERR MAY CONTAIN ADDITIONAL DIAGNOSTIC INFORMATION')
  500 FORMAT (5I10,2F14.0,2I10)
  505 FORMAT (8F15.0)
  510 FORMAT (
     &' NUMBER OF ESTIMATED PARAMETERS...................:',I5,/, 
     &' NUMBER OF HEAD OBSERVATIONS......................:',I5,/, 
     &' NUMBER OF ALL OTHER OBSERVATIONS.................:',I5,/, 
     &' TOTAL NUMBER OF OBSERVATIONS.....................:',I5,//,
     &' NUMBER OF PRIOR-INFORMATION EQUATIONS............:',I5,/, 
     &' NUMBER OF PRIOR INFORMATION W/ FULL WEIGHT MATRIX:',I5,/, 
     &' CALCULATED ERROR VARIANCE........................:',G12.5,//,
     &' EFFECTIVE CORRELATION (EC).......................:',G12.5,A,//,
     &' TRANSFORM WITH OBS. COV. MATRIX? (YES,NO)=(1,0)..:',I5,/)
  515 FORMAT (/,' SCALED PARAMETER COVARIANCE MATRIX, (X''WX)^-1')
  520 FORMAT (/,6X,
     &        'DEPENDENT VARIABLES COMPUTED WITH OPTIMUM PARAMETERS',/,
     &        4X,2('NO. OBSERVATION',5X,'FOPT',8X))
  521 FORMAT (/,6X,
     &        'INTERVAL VARIABLES COMPUTED WITH OPTIMUM PARAMETERS',/,
     &        4X,2('NO.  INTERVAL  ',5X,'GOPT',8X))
  525 FORMAT (/,21X,'PARAMETERS FOR SAMPLE NO.',I4,/,4X,
     &        2('NO. NAME',12X,'B',9X))
  530 FORMAT (/,9X,'DEPENDENT VARIABLES COMPUTED FOR SAMPLE NO.',I4,/,
     &        4X,2('NO. OBSERVATION',6X,'FC',9X))
  535 FORMAT (/,' SENSITIVITIES FOR OPTIMUM PARAMETERS') 
  555 FORMAT (/,11X,'SQUARE-ROOT OF WEIGHTS FOR SAMPLE INFORMATION',/,
     &        4X,2('NO. OBSERVATION',6X,'W',10X))
  556 FORMAT (/,11X,'SQUARE-ROOT OF WEIGHTS FOR INTERVALS',/,4X,
     &        2('NO.   INTERVAL ',6X,'W',10X))
  580 FORMAT (6F13.0)
  590 FORMAT (/,11X,'OBSERVED VALUES OF THE DEPENDENT VARIABLES',/,4X,
     &        2('NO. OBSERVATION',5X,'FOBS',8X))
  595 FORMAT (8F13.0)
  600 FORMAT (/,' MULTIPLE PRIOR NUMBER',I3,', SQUARE-ROOT OF WEIGHT=',
     &        G10.3)
  615 FORMAT (8I10)
  620 FORMAT (/,9X,'PARAMETER',/,3X,'NO.',5X,'NAME',7X,'LN')
  625 FORMAT (1X,I5,3X,A,3X,I2)
  630 FORMAT (6(A12,1X))
  635 FORMAT (' END OF BEALE2.DAT',/,'  MODFLOW-2000 SOLUTION MUST NOT',
     &        ' HAVE CONVERGED FOR NEXT SET OF PARAMETER VALUES.',/,
     &        ' CHECK PARAMETER VALUES; NEGATIVE VALUES CAN BE MADE TO',
     &        ' STAY POSITIVE BY ESTIMATING THE LOG TRANSFORM')
  636 FORMAT (' END OF BEALE4.DAT',/,'  MODFLOW-2000 SOLUTION MUST NOT',
     &        ' HAVE CONVERGED FOR NEXT SET OF PARAMETER VALUES.',/,
     &        ' CHECK PARAMETER VALUES; NEGATIVE VALUES CAN BE MADE TO',
     &        ' STAY POSITIVE BY ESTIMATING THE LOG TRANSFORM')
  645 FORMAT (6(A10,1X))
  660 FORMAT(/,' OBSERVATIONS WITH A FULL WEIGHT MATRIX:')
  670 FORMAT(/,' PRIOR INFORMATION WITH FULL WEIGHT MATRIX')
  680 FORMAT(/,'   SQUARE-ROOT OF FULL WEIGHT MATRIX:')
  690 FORMAT(3X,'PARAMETERS INVOLVED:')
  700 FORMAT(5X,A,2X,1PG13.6)
  710 FORMAT(//,' TOTAL NONLINEARITY (BNT).......... = ',G11.5,
     &        /,' INTRINSIC NONLINEARITY (BNI)...... = ',G11.5,//)
  715 FORMAT(///,' ****** CORRECTION FACTOR COMPUTED FOR CONFIDENCE'
     &           ' REGION')
  720 FORMAT(///,' ****** CORRECTION FACTORS COMPUTED FOR ',A,
     &           ' INTERVALS')
  730 FORMAT(//,
     &' TOTAL NUMBER OF INTERVALS (NOINT)......................:',I5,/,
     &' NUMBER OF INTERVALS FOR HEAD (NHI).....................:',I5,/,
     &' NUMBER OF INTERVALS FOR HEAD DEPENDENT FLOWS (NQI).....:',I5,/,
     &' NUMBER OF INTERVALS FOR PARAMETERS (NPI)...............:',I5,/,
     &' INTERVAL TYPE (1=CONFIDENCE,2=PREDICTION)..............:',I5,/,
     &' HEAD AND FLOW INTERVALS ON DIFFERENCES (0,1)=(NO,YES)..:',I5)
  740 FORMAT (/,16X,
     &        'COMBINED INTRINSIC NONLINEARITY',/,
     &        4X,2('  INTERVAL NO. ',5X,' BMI',8X))
  750 FORMAT (/,6X,
     &        'COMBINED INTRINSIC NONLINEARITY AS IF F WERE LINEAR ',/,
     &        4X,2('  INTERVAL NO. ',5X,'BMF0',8X))
  760 FORMAT (/,6X,
     &        'COMBINED INTRINSIC NONLINEARITY AS IF G WERE LINEAR ',/,
     &        4X,2('  INTERVAL NO. ',5X,'BMG0',8X))
  800 FORMAT(//' CORRECTION FACTOR FOR N-P (A) ---- = ',G11.5
     $/' SCALING FACTOR (B) --------------- = ',G11.5
     $/' EFFECTIVE CORRELATION (C) -------- = ',G11.5
     $/' (N-P)/(N-A*P) VALUE (FAC) -------- = ',G11.5
     $/' CORRECTION FACTOR (CR) ----------- = ',G11.5)
  810 FORMAT(//' APPROXIMATE N-A*P VALUE (DFB) ---- = ',G11.5
     $/' APPROX. (N-P)/(N-A*P) VALUE (FACB) = ',G11.5
     $/' APPROXIMATE BOUND FOR CR (CRB) --- = ',G11.5)
  820 FORMAT(//'** ',A,' INTERVAL NO. ',I3,', FOR ', A)
  830 FORMAT(/' CORRECTION FACTOR XI FOR C. I. ------ = ',G11.5
     $/' CORRECTION FACTOR (CC) -------------- = ',G11.5)
  840 FORMAT(/' ESTIMATED VARIANCE OF PREDICTION (VE) = ',G11.5
     $/' BOUND FOR XI USING POS. VALUES (XIPB) = ',G11.5
     $/' BOUND FOR XI USING NEG. VALUES (XINB) = ',G11.5
     $/' APPROXIMATE BOUND FOR CC (CCB) ------ = ',G11.5)    
  850 FORMAT(/' TRUE DIAGONAL PREDICTION SECOND MOMENT (VP) = '
     $,G11.5) 
  860 FORMAT(/14X,' TRUE OFF-DIAGONAL PREDICTION SECOND MOMENTS'
     $/3(3X,'NO.',10X,'CP',6X))    
  870 FORMAT(/' CORRECTION FACTOR XI FOR P. I. ------ = ',G11.5
     $/' CORRECTION FACTOR (CP) -------------- = ',G11.5)
  880 FORMAT(/' ESTIMATED VARIANCE OF PREDICTION (VE) = ',G11.5
     $/' BOUND FOR XI USING POS. VALUES (XIPB) = ',G11.5
     $/' BOUND FOR XI USING NEG. VALUES (XINB) = ',G11.5
     $/' APPROXIMATE BOUND FOR CP (CPB) ------ = ',G11.5)    
C     ------------------------------------------------------------------      
C
C     WRITE PROGRAM NAME AND VERSION TO SCREEN
      WRITE (*,*)
      WRITE (*,480) VERSION
c
C     DEFINE INPUT AND OUTPUT UNIT NUMBERS
      IIN1 = 11
      IOUT = 3
      OPEN (IOUT,FILE='CORFAC.ERR',STATUS='UNKNOWN')
C     FIND INPUT FILE NAMES BY READING NAME AND OBS FILES,
C     AND READING OUTNAM
      CALL FINDINP(IOUT,OUTNAM,IIN1)
      IF (OUTNAM.EQ.'NONE') THEN
        WRITE(*,490)
        CLOSE(IOUT)
        STOP
      ELSE
        CLOSE(IOUT,STATUS='DELETE')
      ENDIF
      LENGNAM = NONB_LEN(OUTNAM,80)
C     OPEN FILES
      FN = OUTNAM(1:LENGNAM)//'.#cf'
      OPEN (IOUT,FILE=FN,STATUS='UNKNOWN')
      WRITE (IOUT,480) VERSION
      FN = OUTNAM(1:LENGNAM)//'._cf'
      OPEN (IIN1,FILE=FN,STATUS='OLD',ACCESS='SEQUENTIAL',
     &      FORM='FORMATTED',ERR=2,IOSTAT=IOS)
    2 CONTINUE
      IF (IOS.NE.0) THEN
        LENGNAM = NONB_LEN(FN,84)
        WRITE (*,400) FN(1:LENGNAM)
        STOP
      ENDIF
c
C     READ BASE DATA
      IPR = 0
      READ (IIN1,500) NPE, NOBS, NQ, MPR, IPR, VAR, EC, ITRN
      NH = NOBS - NQ
      NPRI = IPR + MPR
      NTOT = NOBS + NPRI
      IF (ITRN.GT.0) THEN
         ITRN=1
      ELSE
         ITRN=0
      END IF   
      IF (EC.LT.0) THEN
         EC = 0.8
         WRITE (IOUT,510) NPE, NH, NQ, NOBS, MPR, IPR, VAR, EC,
     &    '(WAS RESET BECAUSE IN INPUT FILE EC<0.0)', ITRN
      ELSE IF (EC.GT.1.) THEN
         EC = 0.8
         WRITE (IOUT,510) NPE, NH, NQ, NOBS, MPR, IPR, VAR, EC,
     &    '(WAS RESET BECAUSE IN INPUT FILE EC>1.0)', ITRN
      ELSE
         WRITE (IOUT,510) NPE, NH, NQ, NOBS, MPR, IPR, VAR, EC, '',ITRN
      END IF   
C
C     ALLOCATE MEMORY FOR ARRAYS
      ALLOCATE(PARNAM(NPE), OBSNAM(NOBS), X(NPE,NTOT),
     &         WH(NH),WQ(NQ,NQ),WP(MPR),WTP(IPR,IPR),NIPR(IPR),
     &         CF(NOBS,NOBS),D(NPE,NTOT))
      NQD=NQ
      IPD=IPR
      NPD=NPE
      DO 10 I = NOBS+MPR+1, NOBS+MPR+IPR
         DO 5 IP = 1, NPE
            X(IP,I) = 0.0
    5    CONTINUE
   10 CONTINUE
C      
C     READ PARAMETER NAMES
      READ (IIN1,645) (PARNAM(I),I=1,NPE)
C     READ OBSERVATION NAMES
      READ (IIN1,630) (OBSNAM(I),I=1,NOBS)
C     READ AND WRITE SQUARE-ROOT OF WEIGHTS FOR HEADS
      WRITE (IOUT,555)
      IF (NH.GT.0) THEN
        READ (IIN1,505) (WH(I),I=1,NH)
        WRITE (IOUT,*) ' HEADS:'
        CALL PRTOTB12(WH,OBSNAM,NH)
      ENDIF
C     READ AND WRITE FULL SQUARE-ROOT WEIGHT MATRIX ON OBSERVATIONS
      IF (NQ.GT.0) THEN
        DO 15 I = 1, NQ
          READ (IIN1,505) (WQ(I,J),J=1,NQ)
   15   CONTINUE
        WRITE (IOUT,660)
        CALL PRTOTD(WQ,NQ,NQ,NQD,OBSNAM,NH)
      ENDIF
C     READ AND WRITE SENSITIVITIES FOR OBSERVATIONS
      DO 20 J = 1, NOBS
        READ (IIN1,580) (X(I,J),I=1,NPE)
   20 CONTINUE
      WRITE (IOUT,535)
      CALL PRTOTX(X,NPE,NOBS,NPD,OBSNAM,PARNAM)
C     READ AND WRITE PRIOR INFORMATION FROM EQUATIONS
      IF (MPR.GT.0) THEN
        DO 30 I = 1, MPR
          IMP=NOBS+I
          READ (IIN1,505) (X(IP,IMP),IP=1,NPE), WP(I)
          WRITE (IOUT,600) I, WP(I)
          CALL PRTOTB10(X(1,IMP),PARNAM,NPE)
   30   CONTINUE
      ENDIF
C     READ AND WRITE PRIOR INFORMATION WITH FULL WEIGHT MATRIX
      IF (IPR.GT.0) THEN
        WRITE (IOUT,670)
        READ (IIN1,615) (NIPR(I),I=1,IPR)
        WRITE (IOUT,690)
        WRITE (IOUT,700) (PARNAM(NIPR(I)),I=1,IPR)
        J = NOBS + MPR
        DO 35 I = 1, IPR
          READ (IIN1,580) (WTP(IP,I),IP=1,IPR)
          X(NIPR(I), J+I) = 1.0
   35   CONTINUE
        WRITE (IOUT,680)
        CALL PRTOTP(WTP,IPR,IPR,IPD,PARNAM,NIPR,IPD,NPD)
      ENDIF
C      
C     READ BASE DATA FOR INTERVALS
      READ (IIN1,'(6I10)') NOINT, NHI, NQI, NPI, ITYP, IDIF
      IF (NOINT.GT.0) THEN
C
C        ALLOCATE MEMORY FOR ARRAYS
         ALLOCATE(GNAM(NOINT),WG(NOINT),Z(NPE,NOINT),Q(NTOT))
C         
C        READ NAME OF PREDICTIONS
         READ (IIN1,630) (GNAM(I),I=1,NOINT)
C        READ SQUARE-ROOT OF WEIGHTS FOR PREDICTIONS FOR WHICH INTERVAL
C        CORRECTION IS TO BE COMPUTED
         READ(IIN1,505) (WG(I),I=1,NOINT)
         IF (ITYP.NE.2) THEN
            DO 40 I=1,NOINT
               WG(I)=0.
   40       CONTINUE       
         END IF   
         WRITE(IOUT,556)
         CALL PRTOTB12(WG,GNAM,NOINT)
C        READ SENSITIVITIES OF PREDICTIONS WITH RESPECT TO PARAMETERS
         DO 45 I=1,NOINT
            READ(IIN1,580) (Z(IP,I),IP=1,NPE)
   45    CONTINUE           
         WRITE (IOUT,535)
         CALL PRTOTX(Z,NPE,NOINT,NPD,GNAM,PARNAM)
      END IF
C
C        IF TRANSFORM WITH FULL OBSERVATION SECOND-MOMENT MATRIX, FIRST 
C        ALLOCATE MEMORY FOR V AND CP, THEN READ V AND TRANSFORM WITH 
C        WEIGHTS
      IF (ITRN.GT.0) THEN
         ALLOCATE(V(NOBS,NOBS))
         IF (NOINT.GT.0.AND.ITYP.EQ.2) ALLOCATE(CP(NOBS))
C
         DO I=1,NOBS
            READ (IIN1,*) (V(I,J),J=1,NOBS)
         END DO
C
         CALL TRANSV(NH,NQ,V,WH,WQ)
      END IF
C
C        TRANSFORM SENSITIVITIES BY SQUARE-ROOT WEIGHTS
      CALL TRANSX(NTOT,NOBS,NH,NQ,MPR,IPR,NPE,X,WH,WQ,WP,WTP,NIPR,D)
C
C        COMPUTE THE OBSERVATION BLOCK OF THE HAT MATRIX (CF), THE TRACE
C        AND SUM OF ELEMENTS OF I-CF (TOCF AND CFSM), AND THE TRACE OF
C        THE PRIOR BLOCK OF THE I-HAT MATRIX (TPCF)
      CALL HATMAT(NPE,NTOT,NOBS,NPRI,CF,D,X,CFSM,TOCF,TPCF)
      DF = NTOT - NPE
C
C***** FOR CONFIDENCE REGION
C        COMPUTE CORRECTION FOR CONFIDENCE REGION
      WRITE(IOUT,715)
      IF (ITRN.GT.0) THEN
C           COMPUTE B
         B = 0.
         DO 50 I = 1, NOBS
            B = B + V(I,I)
   50    CONTINUE 
         B = (B + NPRI) / NTOT
C           COMPUTE A
         A = NPRI - TPCF
         DO 70 I = 1, NOBS
            DO 60 J = 1, NOBS
               A = A + CF(I,J) * V(J,I)
   60       CONTINUE 
   70    CONTINUE 
         A = A / (B * NPE)
C           COMPUTE EFFECTIVE CORRELATION, COR
         COR = (A - 1.) * NPE / (TOCF - CFSM)
C           COMPUTE VARIANCE AMPLIFICATION RATIO, FAC
         FAC = DF / (NTOT - A * NPE)
C           COMPUTE CORRECTION FACTOR FOR CONFIDENCE REGION, CCR
         CCR = A * FAC
C           PRINT RESULTS
         WRITE(IOUT,800) A, B, COR, FAC, CCR
      END IF   
C
C        COMPUTE APPROXIMATE CORRECTION FOR CONFIDENCE REGION
      OMEC = 1. - EC
C        COMPUTE APPROXIMATE NTOT-A*NPE=DFB
      DFB = OMEC * TOCF + EC * CFSM + TPCF
C        COMPUTE VARIANCE AMPLIFICATION RATIO, FAC
      FACB = DF / DFB
C        COMPUTE BOUND, CRB, FOR CORRECTION FACTOR, CCR
      CRB = FACB * (NPE + EC *(TOCF - CFSM)) / NPE
C        PRINT RESULTS
      WRITE(IOUT,810) DFB, FACB, CRB
C
C***** LOOP ON PROBABILITY INTERVALS
      IF (NOINT.GT.0) THEN
         IF (ITYP.EQ.2) THEN
            WRITE(IOUT,720) 'PREDICTION'
         ELSE
            WRITE(IOUT,720) 'CONFIDENCE'
         END IF
         WRITE (IOUT,730) NOINT, NHI, NQI, NPI, ITYP, IDIF
         WRITE(IOUT,556)
         CALL PRTOTB12(WG,GNAM,NOINT)
         WRITE (IOUT,535)
         CALL PRTOTX(Z,NPE,NOINT,NPD,GNAM,PARNAM)
      END IF   
C      
      DO 170 KNT = 1, NOINT
         IF (ITYP.EQ.2) THEN
            WRITE(IOUT,820) 'PREDICTION', KNT, GNAM(KNT)
         ELSE
            WRITE(IOUT,820) 'CONFIDENCE', KNT, GNAM(KNT)
         END IF
C           COMPUTE Q=W*X*(X'*W*X)^-1*Z' AND VE=Q'*Q
         VE = 0.
         DO 90 I = 1, NTOT
            SUM = 0.
            DO 80 IP = 1, NPE
               SUM = SUM + Z(IP,KNT)*D(IP,I)
   80       CONTINUE 
            VE = VE + SUM * SUM
            Q(I) = SUM
   90    CONTINUE
C           COMPUTE SUM OF SQUARED Q-VALUES FOR PRIOR 
         SSQP = 0.
         IF (NPRI.GT.0) THEN
            DO 100 I = NOBS+1, NTOT
               SSQP = SSQP + Q(I) * Q(I)
  100       CONTINUE
         END IF   
C           COMPUTE QQUD=Q'*V*Q
         IF (ITRN.GT.0) THEN
            QQUD = SSQP
            DO 120 I=1,NOBS
               SUM = 0.
               DO 110 J=1,NOBS
                  SUM = SUM + V(I,J) * Q(J)
  110          CONTINUE
               QQUD = QQUD + SUM * Q(I)
  120       CONTINUE
         END IF
C           FOR OBSERVATION BLOCK COMPUTE SUM OF Q (QSUM), SUM OF 
C           POSITIVES (QSMP), SUM OF NEGATIVES (QSMN), AND SUM OF
C           SQUARED Q VALUES
         QSUM = 0.
         QSMP = 0.
         DO 130 I = 1, NOBS
            QSUM = QSUM + Q(I)
            IF (Q(I).GT.0.) QSMP = QSMP + Q(I)
  130    CONTINUE
         QSMN = QSUM - QSMP
         SSQM = VE - SSQP
C
C        *** FOR CONFIDENCE INTERVALS ***
         IF (ITYP.LT.2) THEN
            IF (ITRN.GT.0) THEN
C                 COMPUTE XI AND CORRECTION FACTOR, CCI
               XI = QQUD / (B * VE)
               CCI = XI * FAC
               WRITE(IOUT,830) XI, CCI
            END IF
C              APPROXIMATE BOUND FOR CCI, CCIB
            XIPB = QSMP * QSMP
            XINB = QSMN * QSMN
            XIMX = MAX(XIPB, XINB)
            CCIB = FACB * (OMEC * SSQM + EC * XIMX + SSQP) / VE
            XIPB = XIPB / VE
            XINB = XINB / VE
            VE = VAR * VE
            WRITE(IOUT,840) VE, XIPB, XINB, CCIB
C
C        *** FOR PREDICTION INTERVALS ***
         ELSE
            IF (ITRN.GT.0) THEN
C                 READ TRUE PREDICTION SECOND MOMENT (VP) AND 
C                 OFFF-DIAGONAL SECOND MOMENTS (CP)
               READ (IIN1,*) VP, (CP(I),I=1,NOBS)
               WRITE(IOUT,850) VP
               WRITE(IOUT,860)
               CALL PRTOTB12(CP,OBSNAM,NOBS)
C                 TRANSFORM CP TO CP*W
               CALL TRANSCP(NOBS,NH,NQ,CP,WH,WQ)
               COVP = 0.
               DO 140 I = 1, NOBS
                  COVP = COVP + CP(I) * Q(I)
  140          CONTINUE 
C                 COMPUTE XI AND CORRECTION FACTOR, CPI
               VD = 1. / (WG(KNT) * WG(KNT))
               XI = (VP + (QQUD - 2. * COVP) / B) / (VE + VD)
               CPI = XI * FAC
               WRITE(IOUT,870) XI, CPI
            ELSE
               VP = 1. / (WG(KNT) * WG(KNT)) 
            END IF
C              APPROXIMATE BOUND FOR CPI, CPIB
            XIPB = QSMP * QSMP
            XINB = QSMN * QSMN - 2. * QSMN * SQRT(VP) + VP
            XIMX = MAX(XIPB, XINB)
            CPIB = FACB * (OMEC * (SSQM + VP) + EC * XIMX + SSQP) / 
     &             (VE + VP)
            XIPB = XIPB / (VE + VP)
            XINB = XINB / (VE + VP)
            VE = VAR * VE
            WRITE(IOUT,880) VE, XIPB, XINB, CPIB
         END IF
  170  CONTINUE
C
      END
C=======================================================================
      SUBROUTINE PRTOTB10(VAL,VID,NO)
C**PRINT VALUES IN TWO GROUPS OF THREE COLUMNS
      INTEGER IIN1, IOUT, K, L, NO, NR
      REAL VAL
      CHARACTER*10 VID(NO)
      DIMENSION VAL(NO)
      COMMON /ITP   / IIN1, IOUT
      NR = NO/2
      IF (2*NR.NE.NO) NR = NR + 1
      DO 10 K = 1, NR
        WRITE (IOUT,500) (L,VID(L),VAL(L),L=K,NO,NR)
   10 CONTINUE
      RETURN
  500 FORMAT (3X,2(I3,2X,A,1X,G11.5,3X))
      END
C=======================================================================
      SUBROUTINE PRTOTB12(VAL,VID,NO)
C**PRINT VALUES IN TWO GROUPS OF THREE COLUMNS
      INTEGER IIN1, IOUT, K, L, NO, NR
      REAL VAL
      CHARACTER*12 VID(NO)
      DIMENSION VAL(NO)
      COMMON /ITP   / IIN1, IOUT
      NR = NO/2
      IF (2*NR.NE.NO) NR = NR + 1
      DO 10 K = 1, NR
        WRITE (IOUT,500) (L,VID(L),VAL(L),L=K,NO,NR)
   10 CONTINUE
      RETURN
  500 FORMAT (3X,2(I3,2X,A,1X,G11.5,3X))
      END
C=======================================================================
      SUBROUTINE PRTOTD(W,NR,NC,NRD,OBSNAM,NH)
C     PRINT A FULL WEIGHT MATRIX FOR OBSERVATIONS DIVIDED VERTICALLY
C     INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NH, NR, NRD
      REAL W(NRD,NC)
      CHARACTER*12 OBSNAM(NC+NH)
      COMMON /ITP   / IIN1, IOUT
C
  500 FORMAT (/,1X,'OBSERVATION',2X,5(1X,A12))
  505 FORMAT (1X,A12,1X,5(1X,G12.5))
  510 FORMAT (1X,12('-'),1X,5(1X,A12))
C
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (OBSNAM(J),J=K+NH,J10+NH)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) OBSNAM(I+NH), (W(I,J),J=K,J10)
   10   CONTINUE
   20 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE PRTOTP(W,NR,NC,NRD,PARNAM,NIPR,IPD,NPD)
C     PRINT A FULL WEIGHT MATRIX FOR PRIOR INFORMATION DIVIDED
C     VERTICALLY INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NR, NRD, IPD, NPD
      INTEGER NIPR(IPD)
      REAL W(NRD,NC)
      CHARACTER*10 PARNAM(NPD)
      COMMON /ITP   / IIN1, IOUT
C
  500 FORMAT (/,1X,'PARAMETER',2X,5(2X,A10,1X))
  505 FORMAT (1X,A10,1X,5(1X,G12.5))
  510 FORMAT (1X,10('-'),1X,5(1X,A12))
C
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (PARNAM(NIPR(J)),J=K,J10)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) PARNAM(NIPR(I)), (W(I,J),J=K,J10)
   10   CONTINUE
   20 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE PRTOTX(X,NR,NC,NRD,OBSNAM,PARNAM)
C**PRINT MATRICES DIVIDED VERTICALLY INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NR, NRD
      REAL X(NRD,NC)
      CHARACTER*10 PARNAM(NR)
      CHARACTER*12 OBSNAM(NC)
      COMMON /ITP   / IIN1, IOUT
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (OBSNAM(J),J=K,J10)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) PARNAM(I), (X(I,J),J=K,J10)
   10   CONTINUE
   20 CONTINUE
  500 FORMAT (/,1X,'PARAMETER',3X,5(A12,1X))
  505 FORMAT (1X,A10,2X,5(G12.5,1X))
  510 FORMAT (1X,10('-'),2X,5(A12,1X))
      RETURN
      END
C=======================================================================
      SUBROUTINE PRTOTC(X,NR,NC,NRD,PARNAM)
C**PRINT MATRICES DIVIDED VERTICALLY INTO FIVE-COLUMN BLOCKS
      INTEGER I, IIN1, IOUT, J, J10, K, NC, NR, NRD
      DOUBLE PRECISION X(NRD,NC)
      CHARACTER*10 PARNAM(NR)
      COMMON /ITP   / IIN1, IOUT
      DO 20 K = 1, NC, 5
        J10 = K + 4
        IF (J10.GT.NC) J10 = NC
        WRITE (IOUT,500) (PARNAM(J),J=K,J10)
        WRITE (IOUT,510) ('------------',J=K,J10)
        DO 10 I = 1, NR
          WRITE (IOUT,505) PARNAM(I), (REAL(X(I,J)),J=K,J10)
   10   CONTINUE
   20 CONTINUE
  500 FORMAT (/,1X,'PARAMETER',3X,5(2X,A10,1X))
  505 FORMAT (1X,A10,2X,5(G12.5,1X))
  510 FORMAT (1X,10('-'),2X,5(A10,1X))
      RETURN
      END
C=======================================================================
      INTEGER FUNCTION NONB_LEN(CHARVAR,LENGTH)
C
C FUNCTION TO RETURN NON-BLANK LENGTH OF CONTENTS OF A CHARACTER VARIABLE
C
C Variable list:
C   CHARVAR  = CHARACTER VARIABLE OF INTEREST
C   LENGTH   = DIMENSIONED LENGTH OF CHARVAR
C
      CHARACTER*(*) CHARVAR,C*1
      INTEGER LENGTH, K
C
      C = ' '
      K = LENGTH+1
C
      DO 20 WHILE (C .EQ. ' ' .AND. K .GE. 1)
        K = K-1
        C = CHARVAR(K:K)
 20   CONTINUE
C
      NONB_LEN = K
      RETURN
      END
C=======================================================================
      SUBROUTINE FINDINP(IOUT,OUTNAM,INWR)
C
      CHARACTER*80 NAMFIL, OBFNAM, OUTNAM, OUTNAMU
      CHARACTER*200 LINE
      LOGICAL EXISTS
      INTEGER INWR, IOUT, ISTAT, NC, N, ISTART, ISTOP, IU, NLEN,
     &        NONB_LEN, LLOC
      REAL R
C
C     NAMFIL is the name of the name file
C     OBFNAM is the name of the OBS file
C     OUTNAM is the name of the output filename base listed in
C            the OBS file
C     OUTNAMU is an upper-case version of OUTNAM, for comparison
C            with 'NONE'
C
  500 FORMAT(' Enter the name of the NAME file:',/)
  510 FORMAT(' The NAME file is: ',A)
  520 FORMAT(A)
  530 FORMAT(/,' Error reading NAME file.  Enter blank line to quit.',/)
  540 FORMAT(/,' Error opening NAME file.  Enter blank line to quit.',/)
  550 FORMAT(/,' Error opening file "',A,'"')
C
    1 CONTINUE
      NAMFIL = ' '
      ISTAT = 0
C *** The following statement should be uncommented in order to use
C *** GETARG to retrieve a command line argument.
      CALL GETARG(1,NAMFIL)
      IF(NAMFIL.NE.' ') THEN
        INQUIRE (FILE=NAMFIL,EXIST=EXISTS)
        IF(.NOT.EXISTS) THEN
          NC=INDEX(NAMFIL,' ')
          NAMFIL(NC:NC+3)='.nam'
          INQUIRE (FILE=NAMFIL,EXIST=EXISTS)
          IF(.NOT.EXISTS) THEN
            WRITE(*,*) ' Name file does not exist'
            STOP
          ENDIF
        ENDIF
      ELSE
        WRITE(*,500)
        READ(*,520,ERR=5,IOSTAT=ISTAT) NAMFIL
      ENDIF
      IF (NAMFIL.EQ.' ') STOP
    5 CONTINUE
      IF (ISTAT.NE.0) THEN
        WRITE(*,530)
        GOTO 1
      ENDIF
      OPEN(INWR,FILE=NAMFIL,STATUS='OLD',ERR=7,IOSTAT=ISTAT)
    7 CONTINUE
      IF (ISTAT.NE.0) THEN
        WRITE(*,540)
        GOTO 1
      ENDIF
      WRITE(*,510) NAMFIL
C
      OUTNAM=' '
   10 CONTINUE
C       Read down in NAME file to find name of OBS file
        CALL URDCOM(INWR,IOUT,LINE,ISTAT)
        IF (ISTAT.EQ.1) THEN
C         END-OF-FILE reached
          OUTNAM='NONE'
          GOTO 40
        ENDIF
        LLOC = 1
C       Look for OBS file type in the name file
        CALL URWORD(LINE,LLOC,ISTART,ISTOP,1,N,R,IOUT,INWR)
        IF (LINE(ISTART:ISTOP).EQ.'OBS') THEN
C         Assign OBFNAM as the name of the OBS file, read from the NAME
C         file
          CALL URWORD(LINE,LLOC,ISTART,ISTOP,2,IU,R,IOUT,INWR)
          CALL URWORD(LINE,LLOC,ISTART,ISTOP,0,N,R,IOUT,INWR)
          OBFNAM = LINE(ISTART:ISTOP)
          CLOSE(INWR)
          OPEN(INWR,FILE=OBFNAM,STATUS='OLD',ERR=20,IOSTAT=ISTAT)
   20     CONTINUE
          IF (ISTAT.NE.0) THEN
            NLEN = NONB_LEN(OBFNAM,80)
            WRITE(*,550) OBFNAM(1:NLEN)
            STOP
          ENDIF
C         Assign OUTNAM as filename base read from OBS file
          CALL URDCOM(INWR,IOUT,LINE,ISTAT)
          LLOC = 1
          CALL URWORD(LINE,LLOC,ISTART,ISTOP,0,N,R,IOUT,INWR)
          OUTNAM = LINE(ISTART:ISTOP)
          CLOSE(INWR)
          CALL UCASE(OUTNAM,OUTNAMU,1)
          IF (OUTNAMU.EQ.'NONE') OUTNAM = 'NONE'
          GOTO 40
        ENDIF
      GOTO 10
C
   40 CONTINUE
      RETURN
      END
C=======================================================================
      SUBROUTINE URDCOM(IN,IOUT,LINE,ISTAT)
C
C-----VERSION 02FEB1999 URDCOM
C     ******************************************************************
C     READ COMMENTS FROM A FILE AND PRINT THEM.  RETURN THE FIRST LINE
C     THAT IS NOT A COMMENT
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*(*) LINE
      INTEGER IN, IOUT, ISTAT, L, I
C     ------------------------------------------------------------------
      ISTAT=0
   10 READ(IN,'(A)',END=40,ERR=50) LINE
      IF(LINE(1:1).NE.'#') RETURN
      L=LEN(LINE)
      IF(L.GT.79) L=79
      DO 20 I=L,1,-1
      IF(LINE(I:I).NE.' ') GO TO 30
   20 CONTINUE
   30 WRITE(IOUT,'(1X,A)') LINE(1:I)
      GO TO 10
C
C     FLAG END OF FILE
   40 CONTINUE
      ISTAT=1
      RETURN
C
C     FLAG READ ERROR
   50 CONTINUE
      ISTAT=2
      RETURN
      END
C=======================================================================
      SUBROUTINE URWORD(LINE,ICOL,ISTART,ISTOP,NCODE,N,R,IOUT,IN)
C
C
C-----VERSION 1003 05AUG1992 URWORD
C     ******************************************************************
C     ROUTINE TO EXTRACT A WORD FROM A LINE OF TEXT, AND OPTIONALLY
C     CONVERT THE WORD TO A NUMBER.
C        ISTART AND ISTOP WILL BE RETURNED WITH THE STARTING AND
C          ENDING CHARACTER POSITIONS OF THE WORD.
C        THE LAST CHARACTER IN THE LINE IS SET TO BLANK SO THAT IF ANY
C          PROBLEMS OCCUR WITH FINDING A WORD, ISTART AND ISTOP WILL
C          POINT TO THIS BLANK CHARACTER.  THUS, A WORD WILL ALWAYS BE
C          RETURNED UNLESS THERE IS A NUMERIC CONVERSION ERROR.  BE SURE
C          THAT THE LAST CHARACTER IN LINE IS NOT AN IMPORTANT CHARACTER
C          BECAUSE IT WILL ALWAYS BE SET TO BLANK.
C        A WORD STARTS WITH THE FIRST CHARACTER THAT IS NOT A SPACE OR
C          COMMA, AND ENDS WHEN A SUBSEQUENT CHARACTER THAT IS A SPACE
C          OR COMMA.  NOTE THAT THESE PARSING RULES DO NOT TREAT TWO
C          COMMAS SEPARATED BY ONE OR MORE SPACES AS A NULL WORD.
C        FOR A WORD THAT BEGINS WITH "'", THE WORD STARTS WITH THE
C          CHARACTER AFTER THE QUOTE AND ENDS WITH THE CHARACTER
C          PRECEDING A SUBSEQUENT QUOTE.  THUS, A QUOTED WORD CAN
C          INCLUDE SPACES AND COMMAS.  THE QUOTED WORD CANNOT CONTAIN
C          A QUOTE CHARACTER.
C        IF NCODE IS 1, THE WORD IS CONVERTED TO UPPER CASE.
C        IF NCODE IS 2, THE WORD IS CONVERTED TO AN INTEGER.
C        IF NCODE IS 3, THE WORD IS CONVERTED TO A REAL NUMBER.
C        NUMBER CONVERSION ERROR IS WRITTEN TO UNIT IOUT IF IOUT IS
C          POSITIVE; ERROR IS WRITTEN TO DEFAULT OUTPUT IF IOUT IS 0;
C          NO ERROR MESSAGE IS WRITTEN IF IOUT IS NEGATIVE.
C     ******************************************************************
C
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER*(*) LINE
      CHARACTER*20 RW,STRING
C     ------------------------------------------------------------------
C
C1------Set last char in LINE to blank and set ISTART and ISTOP to point
C1------to this blank as a default situation when no word is found.  If
C1------starting location in LINE is out of bounds, do not look for a
C1------word.
      LINLEN=LEN(LINE)
      LINE(LINLEN:LINLEN)=' '
      ISTART=LINLEN
      ISTOP=LINLEN
      LINLEN=LINLEN-1
      IF(ICOL.LT.1 .OR. ICOL.GT.LINLEN) GO TO 100
C
C2------Find start of word, which is indicated by first character that
C2------is not a blank and not a comma.
      DO 10 I=ICOL,LINLEN
      IF(LINE(I:I).NE.' ' .AND. LINE(I:I).NE.',') GO TO 20
10    CONTINUE
      ICOL=LINLEN+1
      GO TO 100
C
C3------Found start of word.  Look for end.
C3A-----When word is quoted, only a quote can terminate it.
20    IF(LINE(I:I).EQ.'''') THEN
         I=I+1
         IF(I.LE.LINLEN) THEN
            DO 25 J=I,LINLEN
            IF(LINE(J:J).EQ.'''') GO TO 40
25          CONTINUE
         END IF
C
C3B-----When word is not quoted, space or comma will terminate.
      ELSE
         DO 30 J=I,LINLEN
         IF(LINE(J:J).EQ.' ' .OR. LINE(J:J).EQ.',') GO TO 40
30       CONTINUE
      END IF
C
C3C-----End of line without finding end of word; set end of word to
C3C-----end of line.
      J=LINLEN+1
C
C4------Found end of word; set J to point to last character in WORD and
C-------set ICOL to point to location for scanning for another word.
40    ICOL=J+1
      J=J-1
      IF(J.LT.I) GO TO 100
      ISTART=I
      ISTOP=J
C
C5------Convert word to upper case and RETURN if NCODE is 1.
      IF(NCODE.EQ.1) THEN
         IDIFF=ICHAR('a')-ICHAR('A')
         DO 50 K=ISTART,ISTOP
            IF(LINE(K:K).GE.'a' .AND. LINE(K:K).LE.'z')
     1             LINE(K:K)=CHAR(ICHAR(LINE(K:K))-IDIFF)
50       CONTINUE
         RETURN
      END IF
C
C6------Convert word to a number if requested.
100   IF(NCODE.EQ.2 .OR. NCODE.EQ.3) THEN
         RW=' '
         L=20-ISTOP+ISTART
         IF(L.LT.1) GO TO 200
         RW(L:20)=LINE(ISTART:ISTOP)
         IF(NCODE.EQ.2) READ(RW,'(I20)',ERR=200) N
         IF(NCODE.EQ.3) READ(RW,'(F20.0)',ERR=200) R
      END IF
      RETURN
C
C7------Number conversion error.
200   IF(NCODE.EQ.3) THEN
         STRING= 'A REAL NUMBER'
         L=13
      ELSE
         STRING= 'AN INTEGER'
         L=10
      END IF
C
C7A-----If output unit is negative, set last character of string to 'E'.
      IF(IOUT.LT.0) THEN
         N=0
         R=0.
         LINE(LINLEN+1:LINLEN+1)='E'
         RETURN
C
C7B-----If output unit is positive; write a message to output unit.
      ELSE IF(IOUT.GT.0) THEN
         IF(IN.GT.0) THEN
            WRITE(IOUT,201) IN,LINE(ISTART:ISTOP),STRING(1:L),LINE
         ELSE
            WRITE(IOUT,202) LINE(ISTART:ISTOP),STRING(1:L),LINE
         END IF
201      FORMAT(1X,/1X,'FILE UNIT',I4,' : ERROR CONVERTING "',A,
     1       '" TO ',A,' IN LINE:',/1X,A)
202      FORMAT(1X,/1X,'KEYBOARD INPUT : ERROR CONVERTING "',A,
     1       '" TO ',A,' IN LINE:',/1X,A)
C
C7C-----If output unit is 0; write a message to default output.
      ELSE
         IF(IN.GT.0) THEN
            WRITE(*,201) IN,LINE(ISTART:ISTOP),STRING(1:L),LINE
         ELSE
            WRITE(*,202) LINE(ISTART:ISTOP),STRING(1:L),LINE
         END IF
      END IF
C
C7D-----STOP after writing message.
      STOP
      END
C=======================================================================
      SUBROUTINE UCASE(WORDIN,WORDOUT,ICASE)
C     VERSION 19980316 ERB
C     ******************************************************************
C     CONVERT A CHARACTER STRING TO ALL UPPER (ICASE > 0) OR ALL
C     LOWER (ICASE < 0) CASE.  IF ICASE = 0, NO CASE CONVERSION IS DONE.
C     ******************************************************************
C       SPECIFICATIONS:
C     ------------------------------------------------------------------
      CHARACTER WORDIN*(*), WORDOUT*(*)
      INTEGER ICASE, LENGNB, LENGIN, LENGOUT
C
C     DETERMINE LENGTH OF STRING VARIABLES
      LENGIN = LEN(WORDIN)
      LENGOUT = LEN(WORDOUT)
C
C     DETERMINE IF WORDOUT IS LONG ENOUGH TO CONTAIN NON-BLANK LENGTH
C     OF WORDIN
      LENGNB = NONB_LEN(WORDIN,LENGIN)
      IF (LENGNB.GT.LENGOUT) STOP 'STRING-LENGTH ERROR IN UCASE'
C
      WORDOUT = WORDIN
      IDIFF=ICHAR('a')-ICHAR('A')
      IF (ICASE.GT.0) THEN
C       CONVERT STRING TO UPPER CASE
        DO 10 K=1,LENGNB
          IF(WORDIN(K:K).GE.'a' .AND. WORDIN(K:K).LE.'z')
     1      WORDOUT(K:K)=CHAR(ICHAR(WORDIN(K:K))-IDIFF)
10      CONTINUE
C
      ELSEIF (ICASE.LT.0) THEN
C       CONVERT STRING TO LOWER CASE
        DO 20 K=1,LENGNB
          IF(WORDIN(K:K).GE.'A' .AND. WORDIN(K:K).LE.'Z')
     1      WORDOUT(K:K)=CHAR(ICHAR(WORDIN(K:K))+IDIFF)
20      CONTINUE
C
      ENDIF
C
      RETURN
      END
C=======================================================================

C     Last change:  SC   30 Oct 2003    4:05 pm
      SUBROUTINE HATMAT(NPE,NTOT,NOBS,NPRI,CF,D,X,CFSM,TOCF,TPCF)
C
C     VERSION 1001 24OCT2003 (SC) 
C     ******************************************************************
C     COMPUTES THE OBSERVATION BLOCK OF THE HAT MATRIX, CF, THE TRACES
C     OF THE OBSERVATION AND PRIOR BLOCKS OF THE I-HAT MATRIX, TOCF AND
C     TPCF, AND THE SUM OF ELEMEMTS OF THE I-HAT MATRIX, CFSM.
C     IN RETURN D=(X'*W*X)^-1*X'*W
C     ******************************************************************
C     SPECIFICATIONS:
C     ------------------------------------------------------------------
      INTEGER NTOT, NOBS, NPE, I, J, K, IP, JP, ISTOP
      REAL TOCF, TPCF, X(NPE,NTOT)
      DOUBLE PRECISION CF(NOBS,NOBS), D(NPE,NTOT), DTMP
      DOUBLE PRECISION, ALLOCATABLE :: C(:,:), SCLE(:)
      REAL SUM
C     ------------------------------------------------------------------
      ALLOCATE (SCLE(NPE), C(NPE,NPE))
C
C     COMPUTE C=X'*X AND FACTOR IT
      DO 10 IP = 1, NPE
         DO 5 JP = 1, NPE
            C(IP,JP)=0.
    5    CONTINUE 
   10 CONTINUE 
      DO 40 I = 1, NTOT
         DO 30 IP = 1, NPE
            DTMP = X(IP,I)
            DO 20 JP = IP, NPE
               C(IP,JP) = C(IP,JP) + DTMP * X(JP,I)
   20       CONTINUE
   30    CONTINUE
   40 CONTINUE
      DO 50 IP = 1, NPE
         SCLE(IP) = DSQRT(C(IP,IP))
   50 CONTINUE
      IF(NPE.GT.1) THEN
         DO 70 IP = 1, NPE-1
            DTMP = SCLE(IP)
            DO 60 JP = IP+1, NPE
               C(IP,JP) = C(IP,JP) / (SCLE(JP) * DTMP)
               C(JP,IP) = C(IP,JP)
   60       CONTINUE
            C(IP,IP)=1.D00
   70    CONTINUE
      ENDIF
      C(NPE,NPE)=1.D00
C     FACTOR C
      CALL SOLVE(C, D, D, NPE, NPE, 1, ISTOP, DTMP)
      IF (ISTOP.GT.0) STOP ' C MATRIX SINGULAR'
C
C     COMPUTE D
      DO 100 I = 1, NTOT
         DO 80 IP = 1, NPE
            D(IP,I) = X(IP,I) / SCLE(IP)
   80    CONTINUE
         CALL SOLVE(C, D(1,I), D(1,I), NPE, NPE, 0, ISTOP, DTMP)
         DO 90 IP = 1, NPE
            D(IP,I) = D(IP,I) / SCLE(IP)
   90    CONTINUE
  100 CONTINUE
C
      DEALLOCATE (SCLE, C)
C
C     COMPUTE CF, TOCF, AND CFSM
      TOCF = 0.
      CFSM = 0.
      DO 160 I = 1, NOBS
         DO 150 J = 1, NOBS
            SUM = 0.
            DO 140 IP = 1, NPE
               SUM = SUM + X(IP,I)*D(IP,J)
  140       CONTINUE
            CF(I,J) = SUM
            CFSM = CFSM + SUM
  150    CONTINUE
         TOCF = TOCF + CF(I,I)
  160 CONTINUE
      TOCF = NOBS - TOCF
      CFSM = NOBS - CFSM
C
C     COMPUTE TPCF
      TPCF = 0.
      DO 180 I = NOBS+1, NTOT
         SUM = 0.
         DO 170 IP = 1, NPE
            SUM = SUM + X(IP,I)*D(IP,I)
  170    CONTINUE
         TPCF = TPCF + SUM
  180 CONTINUE
      TPCF = NPRI - TPCF
C
      RETURN 
      END
C=======================================================================
      SUBROUTINE SOLVE(C,D,G,NPE,NPD,IND,ISTOP,DET)
C-----VERSION 1001 25FEB2003 (SC)
C     ******************************************************************
C         SOLVE SYSTEM OF LINEAR EQUATIONS BY LU-DECOMPOSITION
C     ******************************************************************
C        SPECIFICATIONS:
C     ------------------------------------------------------------------
      DOUBLE PRECISION C(NPD,NPD),D(NPD),G(NPD),DPIV,DTMPA,DSUM,DET
C      REAL DET
      INTEGER ISTOP,IND,NPE,NM1,K,KP1,J,I,IP1,NPD
C     ------------------------------------------------------------------
C         COMPUTE TRIAL PARAMETER STEP LENGTHS USING LDU FACTORIZATION:
C           DECOMPOSE MATRIX
      NM1=NPE-1
      ISTOP=0
C
C-----------LDU FACTORIZATION
      IF (IND.GT.0) THEN
         DET=1.
         DO 30 K=1,NM1
            DPIV=C(K,K)
            DET=DET*DPIV
            IF (DPIV.LT.1.D-13) THEN
               ISTOP=1
               RETURN
            ENDIF
            DPIV=1.D00/DPIV
            KP1=K+1
            DO 20 J=KP1,NPE
               DTMPA=C(J,K)*DPIV
               DO 10 I=J,NPE
                  C(I,J)=C(I,J)-DTMPA*C(I,K)
   10          CONTINUE
   20       CONTINUE
            C(K,K)=DPIV
   30    CONTINUE
         DET=DET*C(NPE,NPE)
         IF(C(NPE,NPE).LT.1.D-13) THEN
            ISTOP=1
            RETURN
         ENDIF
         IF (IND.LT.2) RETURN
      ENDIF
C
C-----------INITIALIZE D
      DO 40 K=1,NPE
         D(K)=G(K)
   40 CONTINUE
C
C-----------FORWARD SUBSTITUTE
      DO 60 K=1,NM1
         DTMPA=D(K)*C(K,K)
         KP1=K+1
         DO 50 J=KP1,NPE
            D(J)=D(J)-C(J,K)*DTMPA
   50    CONTINUE
   60 CONTINUE
C
C-----------BACKWARD SUBSTITUTE
      D(NPE)=D(NPE)/C(NPE,NPE)
      DO 90 I=NPE-1,1,-1
         IP1=I+1
         DSUM=0.
         DO 70 J=IP1,NPE
            DSUM=DSUM+C(J,I)*D(J)
   70    CONTINUE
         D(I)=(D(I)-DSUM)*C(I,I)
   90 CONTINUE
C
      RETURN
      END
C=======================================================================

C     Last change:  SC   24 Oct 2003    8:58 pm
      SUBROUTINE TRANSCP(NOBS,NH,NQ,CP,WH,WQ)
C
C     VERSION 1001 24OCT2003 (SC) 
C     ******************************************************************
C     TRANSFORMS V BY MULTIPLYING TWICE WITH SQUARE-ROOT WEIGHTS TO
C     BECOME W*V*W
C     ******************************************************************
C     SPECIFICATIONS:
C     ------------------------------------------------------------------
      INTEGER NOBS, NH, NQ, I, J, K, L
      REAL WH(NH), WQ(NQ,NQ), CP(NOBS), SUM
      REAL, ALLOCATABLE :: X(:)
C     ------------------------------------------------------------------
C
C
C---- HEADS
      IF (NH .GT. 0) THEN
         DO 10 I = 1, NH
            CP(I) = CP(I) * WH(I)
   10    CONTINUE
      ENDIF
C
C-----FLOWS
      IF (NQ .GT. 0) THEN
         ALLOCATE(X(NQ))
         DO 30 I = 1, NQ
            SUM = 0.
            DO 20 J = 1, NQ
               SUM = SUM + WQ(I,J) * CP(J)
   20       CONTINUE
            X(I) = SUM
   30    CONTINUE
         DO 40 I =1, NQ
            CP(I) = X(I)
   40    CONTINUE
         DEALLOCATE (X)
      ENDIF
C
      RETURN
      END
C     Last change:  SC   24 Oct 2003    8:56 pm
      SUBROUTINE TRANSV(NH,NQ,V,WH,WQ)
C
C     VERSION 1001 24OCT2003 (SC) 
C     ******************************************************************
C     TRANSFORMS V BY MULTIPLYING TWICE WITH SQUARE-ROOT WEIGHTS TO
C     BECOME W*V*W
C     ******************************************************************
C     SPECIFICATIONS:
C     ------------------------------------------------------------------
      INTEGER NH, NQ, I, J, K, L
      REAL WH(NH), WQ(NQ,NQ)
      DOUBLE PRECISION V(NH+NQ,NH+NQ), W, DTMPA
      DOUBLE PRECISION, ALLOCATABLE :: X(:,:)
C     ------------------------------------------------------------------
C
C---- HEADS
      IF (NH .GT. 0) THEN
         DO 50 J=1,NH
            W=DBLE(WH(J))
            DO 40 I=J,NH
               V(J,I)=W*V(J,I)*DBLE(WH(I))
               V(I,J)=V(J,I)
   40       CONTINUE
   50    CONTINUE
      ENDIF
C
C-----FLOWS
      IF (NQ .GT. 0) THEN
         ALLOCATE(X(NQ,NQ))
         DO 60 I = 1, NQ
            DO 55 J = 1, NQ
               X(I,J) = 0.0
   55       CONTINUE
   60    CONTINUE
         DO 100 J = 1, NQ
            DO 90 K = 1, NQ
               DTMPA = 0.0
               DO 70 L = 1, NQ
                  DTMPA = DTMPA + V(NH+K,NH+L)*DBLE(WQ(L,J))
   70          CONTINUE
               DO 80 I = J, NQ
                  X(I,J) = X(I,J) + DBLE(WQ(I,K))*DTMPA
   80          CONTINUE
   90       CONTINUE
  100    CONTINUE
         DO 120 J = 1, NQ
            K = NH + J
            V(K,K) = X(J,J)
            DO 110 I = J+1, NQ
               L = NH + I
               V(L,K) = X(I,J)
               V(K,L) = V(L,K)
  110       CONTINUE
  120    CONTINUE
         DEALLOCATE (X)
      ENDIF
C
      RETURN
      END
C     Last change:  SC   30 Oct 2003    3:54 pm
      SUBROUTINE TRANSX(NTOT,NOBS,NH,NQ,MPR,IPR,NPE,X,WH,WQ,WP,WTP,NIPR,
     &                  D)
C
C     VERSION 1001 24OCT2003 (SC) 
C     ******************************************************************
C     TRANSFORMS X BY MULTIPLYING WITH SQUARE-ROOT WEIGHTS TO BECOME X*W
C     ******************************************************************
C     SPECIFICATIONS:
C     ------------------------------------------------------------------
      INTEGER NTOT, NOBS, NH, NQ, MPR, IPR, NPE, NIPR(IPR), I, IP, J, K
      REAL WH(NH), WQ(NQ,NQ), WP(MPR), WTP(IPR,IPR), X(NPE,NTOT)
      DOUBLE PRECISION D(NPE,NTOT)
      REAL SUM, W
C     ------------------------------------------------------------------
C
C-----HEADS
      IF (NH.GT.0) THEN
         DO 15 I = 1, NH
            W = WH(I)
            DO 10 IP=1,NPE
               X(IP, I) = X(IP, I) * W
   10       CONTINUE
   15    CONTINUE
      END IF
C
C-----FLOWS
      IF (NQ.GT.0) THEN
         DO 40 I = 1, NQ
            DO 30 IP = 1, NPE
               SUM = 0.
               DO 20 J = 1, NQ
                  SUM = SUM + WQ(I,J)*X(IP,NH+J)
   20          CONTINUE 
               D(IP,I) = SUM
   30       CONTINUE
   40    CONTINUE
         DO 60 I=1, NQ
            J = NH + I
            DO 50 IP = 1, NPE
               X(IP,J) = D(IP,I)
   50       CONTINUE 
   60    CONTINUE 
      END IF
C
C-----PRIOR INFORMATION ON PARAMETER SUMS
      IF (MPR.GT.0) THEN
         DO 80 I = 1, MPR
            W = WP(I)
            J = NOBS + I
            DO 70 IP = 1, NPE
               X(IP,J) = X(IP,J) * W
   70       CONTINUE 
   80    CONTINUE 
      END IF
C
C-----FULLY CORRELATED PRIOR INFORMATION
      IF (IPR.GT.0) THEN
         K = NOBS + MPR
         DO 100 I = 1, IPR
            IP = NIPR(I)
            DO 90 J = 1, IPR
               X(IP, K+J) = WTP(J,I)
   90       CONTINUE 
  100    CONTINUE 
      END IF   
C
      RETURN 
      END
